package com.cuit.solarenergy.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.solarenergy.base.annotation.AopLog;
import com.cuit.solarenergy.base.annotation.AuthToken;
import com.cuit.solarenergy.entity.TStreamBaseInfo;
import com.cuit.solarenergy.entity.vo.ret.LegendRet;
import com.cuit.solarenergy.entity.vo.ret.RetResponse;
import com.cuit.solarenergy.entity.vo.ret.RetResult;
import com.cuit.solarenergy.entity.vo.ret.Stream;
import com.cuit.solarenergy.exception.ParamsException;
import com.cuit.solarenergy.mapper.TStreamBaseInfoMapper;
import com.cuit.solarenergy.service.CommonService;
import com.cuit.solarenergy.service.LegendTableService;
import com.cuit.solarenergy.vo.TDataOriginMethod;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-03  14:16
 * @Description: TODO
 * @Version: 1.0
 */
@Api(tags = "2、公共模块")
@ApiSupport(order = 2)
@Slf4j
@RestController()
@Transactional(rollbackFor = Exception.class)
public class CommonController {

    @Resource
    private CommonService commonService;


    @Resource
    private TStreamBaseInfoMapper tStreamBaseInfoMapper;

    @Resource
    private LegendTableService legendTableService;

    /**
     * 方法功能：获取所有流域名称
     *
     * @param
     * @return RetResult<List < String>>
     * @date 2023/6/17 10:04
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 1)
    @PreAuthorize("hasAuthority('common:getAllStreamName')")
    @ApiOperation("1、查询所有流域名称")
    @RequestMapping(value = "/common/getAllStreamName", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<Stream>> getAllStreamName() {
        List<Stream> streamList;
        try {
            streamList = commonService.getAllStreamName();
            return RetResponse.makeOKRsp(streamList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：获取所有反演数据(水质参数)
     *
     * @param
     * @return
     * @date 2023/6/19 14:34
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 2)
    @ApiOperation("2、获取所有反演数据(水质参数)")
    @PreAuthorize("hasAuthority('common:getAllWaterQualityParameters')")
    @RequestMapping(value = "/common/getAllWaterQualityParameters", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> getAllWaterQualityParameters() {
        List<String> stringList;
        try {
            stringList = commonService.getAllWaterQualityParameters();
            return RetResponse.makeOKRsp(stringList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：查询所有的数据获取方式
     *
     * @param
     * @return
     * @date 2023/6/19 15:03
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 3)
    @ApiOperation("3、查询所有的数据获取方式")
    @PreAuthorize("hasAuthority('common:getAllWaterQualityParameters')")
    @RequestMapping(value = "/common/getAllDataOriginal", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<TDataOriginMethod>> getAllDataOriginal() {
        List<TDataOriginMethod> tDataOriginMethodList;
        try {
            tDataOriginMethodList = commonService.getAllDataOriginal();
            return RetResponse.makeOKRsp(tDataOriginMethodList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }


    /**
     * 方法功能：获取原始影像数据集的集合
     *
     * @param
     * @return
     * @date 2023/7/4 9:46
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 4)
    @ApiOperation("4、获取原始影像数据集的集合")
    @PreAuthorize("hasAuthority('common:getOriginalDataSet')")
    @RequestMapping(value = "/common/getOriginalDataSet", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> getOriginalDataSet() {
        List<String> stringList;
        try {
            stringList = commonService.getOriginalDataSet();
            return RetResponse.makeOKRsp(stringList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：通过缩略图查询真实图片URL
     *
     * @param simplePicture
     * @return
     * @date 2023/7/3 15:11
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 5)
    @ApiOperation("5、通过缩略图查询真实图片URL")
    @PreAuthorize("hasAuthority('common:getRealPictureForOne')")
    @RequestMapping(value = "/common/getRealPictureForOne", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "simplePicture", value = "缩略图片URL", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> getRealPictureForOne(String simplePicture) {
        String realPictureURL;
        try {
            realPictureURL = commonService.getRealPictureForOne(simplePicture);
            return RetResponse.makeOKRsp(realPictureURL);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：通过缩略图查询真实视频URL
     *
     * @param simpleVideo
     * @return
     * @date 2023/7/3 15:11
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 6)
    @ApiOperation("6、通过缩略图查询真实视频URL")
    @PreAuthorize("hasAuthority('common:getRealVideoForOne')")
    @RequestMapping(value = "/common/getRealVideoForOne", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "simpleVideo", value = "缩略视频URL", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> getRealVideoForOne(String simpleVideo) {
        String realVideoURL;
        try {
            realVideoURL = commonService.getRealVideoForOne(simpleVideo);
            return RetResponse.makeOKRsp(realVideoURL);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：通过缩略图查询真实无人机URL
     *
     * @param simpleUAV
     * @return
     * @date 2023/7/3 15:11
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 7)
    @ApiOperation("7、通过缩略图查询真实无人机URL")
    @PreAuthorize("hasAuthority('common:getRealUAVForOne')")
    @RequestMapping(value = "/common/getRealUAVForOne", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "simpleUAV", value = "缩略无人机URL", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> getRealUAVForOne(String simpleUAV) {
        String realUAVURL;
        try {
            realUAVURL = commonService.getRealUAVForOne(simpleUAV);
            return RetResponse.makeOKRsp(realUAVURL);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：获取某一流域下所有的站点名称
     *
     * @param streamName
     * @return
     * @date 2023/7/3 15:56
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 8)
    @ApiOperation("8、获取某一流域下所有的站点名称")
    @PreAuthorize("hasAuthority('common:getStationsForOneStream')")
    @RequestMapping(value = "/common/getStationsForOneStream", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "streamName", value = "流域名称", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> getStationsForOneStream(String streamName) {
        List<String> stationsList;
        try {
            stationsList = commonService.getStationsForOneStream(streamName);
            return RetResponse.makeOKRsp(stationsList);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：获取某个站点所属流域名称
     *
     * @param siteId
     * @return
     * @date 2023/7/3 16:08
     * @author Tjb
     */

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 9)
    @ApiOperation("9、获取某个站点所属流域名称")
    @PreAuthorize("hasAuthority('common:getStationForOneSite')")
    @RequestMapping(value = "/common/getStationForOneSite", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "siteId", value = "站点id", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> getStationForOneSite(String siteId) {
        String stream;
        try {
            stream = commonService.getStationForOneSite(siteId);
            return RetResponse.makeOKRsp(stream);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：获取卫星列表
     *
     * @param
     * @return
     * @date 2023/8/9 19:22
     * @author Tjb
     */

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 10)
    @ApiOperation("10、获取卫星列表")
    @PreAuthorize("hasAuthority('common:getSatelliteList')")
    @RequestMapping(value = "/common/getSatelliteList", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<Object>> getSatelliteList() {
        List<Object> list;
        try {
            list = commonService.getSatelliteList();
            return RetResponse.makeOKRsp(list);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * 方法功能：获取流域bbox
     *
     * @param streamCodes
     * @return
     * @date 2023/8/9 19:59
     * @author Tjb
     */

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 11)
    @ApiOperation("11、获取流域bbox")
    @PreAuthorize("hasAuthority('common:getStreamBbox')")
    @RequestMapping(value = "/common/getStreamBbox", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> getStreamBbox(@RequestBody List<String> streamCodes) {
        List<String> streamBboxList;
        try {
            streamBboxList = commonService.getStreamBbox(streamCodes);
            return RetResponse.makeOKRsp(streamBboxList);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(900, e.getMessage());
        }
    }


    /**
     * 方法功能：获取所有流域代码及名称
     *
     * @param
     * @return RetResult<List < String>>
     * @date 2023/6/17 10:04
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 12)
    @PreAuthorize("hasAuthority('common:getAllStreamInfo')")
    @ApiOperation("12、获取所有流域代码及名称")
    @RequestMapping(value = "/common/getAllStreamInfo", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<TStreamBaseInfo>> getAllStreamInfo() {
        List<TStreamBaseInfo> streamList;
        try {
            LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
            streamList = tStreamBaseInfoMapper.selectList(queryWrapper);
            return RetResponse.makeOKRsp(streamList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }


    /**
     * 方法功能：获取站点相对位置列表
     *
     * @param
     * @return RetResult<List < String>>
     * @date 2023/6/17 10:04
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 13)
    @PreAuthorize("hasAuthority('common:getAllSiteLocation')")
    @ApiOperation("13、获取站点相对位置列表")
    @RequestMapping(value = "/common/getAllSiteLocation", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> getAllSiteLocation() {
        List<String> locationList;
        try {
            locationList = commonService.getAllSiteLocation();
            return RetResponse.makeOKRsp(locationList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：获取站点相对距离列表
     *
     * @param
     * @return RetResult<List < String>>
     * @date 2023/6/17 10:04
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 14)
    @PreAuthorize("hasAuthority('common:getAllSiteDistance')")
    @ApiOperation("14、获取站点相对距离列表")
    @RequestMapping(value = "/common/getAllSiteDistance", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<Double>> getAllSiteDistance() {
        List<Double> distanceList;
        try {
            distanceList = commonService.getAllSiteDistance();
            return RetResponse.makeOKRsp(distanceList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");
        }
    }

    /**
     * 方法功能：获取图例数据
     *
     * @param
     * @date 2023/6/17 10:04
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 15)
    @PreAuthorize("hasAuthority('common:getAllSiteDistance')")
    @ApiOperation("15、获取图例数据")
    @RequestMapping(value = "/common/legend", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dataType", value = "数据类型（STA/UVA）", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "dataCode", value = "反演数据名称（叶绿素、．．．．）", required = true, dataTypeClass = String.class, paramType = "query")
    })

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<LegendRet>> getLegend(@NotNull String dataType,@NotNull String dataCode) {
        try {
            List<LegendRet> resList = legendTableService.getLegend(dataType, dataCode);
            return RetResponse.makeOKRsp(resList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
