package com.cuit.solarenergy.controller;

import com.cuit.solarenergy.base.annotation.AopLog;
import com.cuit.solarenergy.base.annotation.AuthToken;
import com.cuit.solarenergy.entity.vo.req.DataQueryGetVoReq;
import com.cuit.solarenergy.entity.vo.ret.*;
import com.cuit.solarenergy.exception.ParamsException;
import com.cuit.solarenergy.service.DataManagerService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * @BelongsProject: railway
 * @BelongsPackage: com.cuit.railway.controller
 * @Author: Luolw
 * @Date: 2023/8/4 9:45
 * @Description:
 */


@Api(tags = "9、数据查询模块")
@ApiSupport(order = 9)
@Slf4j
@RestController()
@Transactional(rollbackFor = Exception.class)
public class DataQueryController {
    @Resource
    private DataManagerService dataManagerService;

    /**
     * @Description: 数据查询-采样点查询
     * @Param: DataQueryGetVoReq
     * @return: DataManageGetVoRetList
     * @Author: LuoSir
     * @Date: 2023/8/4
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 1)
    @ApiOperation("1、数据查询-采样点查询接口")
    @PreAuthorize("hasAuthority('data:query:getSamplePointData')")
    @RequestMapping(value = "/data/query/getSamplePointData", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<StatisticQueryVoRet>> getSamplePointData(@RequestBody DataQueryGetVoReq dataQueryGetVoReq) {
        PageVo<StatisticQueryVoRet> statisticQueryVoRetPageVo;
        try {
            statisticQueryVoRetPageVo = dataManagerService.getSamplePointData(dataQueryGetVoReq);
            return RetResponse.makeOKRsp(statisticQueryVoRetPageVo);
        } catch (ParamsException paramsException) {
            return RetResponse.makeErrRsp(paramsException.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }
}
