package com.cuit.solarenergy.controller;


import com.cuit.solarenergy.base.annotation.AopLog;
import com.cuit.solarenergy.entity.vo.ret.RetResponse;
import com.cuit.solarenergy.entity.vo.ret.RetResult;
import com.cuit.solarenergy.service.impl.LoginServiceImpl;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "1、登录退出模块")
@ApiSupport(order = 1)
@Slf4j
@RestController()
@Transactional(rollbackFor = Exception.class)
public class LoginController {

    @Autowired
    private LoginServiceImpl loginService;


    @AopLog
    @ApiOperationSupport(order = 1)
    @ApiOperation("1、用户登录")
    @RequestMapping(value = "/user/login", method = {RequestMethod.POST})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userName",value = "用户名",required = true),
            @ApiImplicitParam(name = "password",value = "密码",required = true)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult login(String userName,String password){
        //登录
       try{
           return loginService.login(userName,password);
       } catch (Exception e){
           e.printStackTrace();
           return RetResponse.makeRsp(900, e.getMessage());
       }
    }
    @AopLog
    @ApiOperationSupport(order = 2)
    @ApiOperation("2、用户退出")
    @RequestMapping(value = "/user/logout", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult logout(){
        //退出
        return loginService.logout();
    }
}