package com.cuit.solarenergy.entity;

import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 上传的tif文件相关信息
 * @author cuitg -- ZengwuWang
 * @date 2024年08月29日 11:19
 */
@Data
public class TifFileInfo {
    /*
        （1）采样作业数据检验结果数据文件
        CZTL_采集日期_水样数据_生成时间（YYYYMMDD）.xlsx
        采集日期格式：YYYY、YYYYMM或YYYYMMDD
        （2）采样作业现场照片文件
        PIC_河流代码（2位）_点位（2位）_序号（3位）_照片日期

        （3）卫星原始数据文件
        SAT_ORD_河流代码（2位）_点位（2位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：5，分别是蓝、绿、红、红边、近红外。
        如果不分点位，则点位编号为00。
        （4）卫星原始RGB数据文件
        SAT_ORD_RGB_河流代码（2位）_点位（2位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：3，分别是蓝、绿、红。
        如果不分点位，则点位编号为00。
        （5）卫星原始拼接红外文件
        SAT_ORD_IR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：1个波段
        如果不分点位，则点位编号为00。
        成时间为完成该数据处理的时间。
        （6）卫星原始拼接近红外文件
        SAT_ORD_NIR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：1个波段
        （7）卫星反演数据文件
        SAT_RTD_河流代码（2位）_点位（2位）_反演水质参数（2-3位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif
        如果不分点位，则点位编号为00。
        生成时间为完成该数据处理的时间。

        （8）无人机原始拼接DOM文件
        UAV_ORD_DOM_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：3个波段或4个波段，分别是蓝、绿、红+(alpha)
        （9）无人机原始拼接红外文件
        UAV_ORD_IR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：1个波段
        如果不分点位，则点位编号为00。
        成时间为完成该数据处理的时间。
        （10）无人机原始拼接近红外文件
        UAV_ORD_NIR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif
        波段数量：1个波段
        如果不分点位，则点位编号为00。
        成时间为完成该数据处理的时间。
        （11）无人机反演数据文件
        UAV_RTD_河流代码（2位）_点位（2位）_反演水质参数（2-3位）_数据日期（YYYYMMDDHH）_生成时间（YYYYMMDDhhmm）.tif
        其中：
        河流代码（两位字符，大写）：
        帕隆藏布：PL
        怒江：NJ
        金沙江：JS
        大渡河：LD
        反演水质参数：
        悬浮物：SPM
        叶绿素浓度：CPC
        透明度：TSP
        总氮：TNG
        浊度：TBD
        营养状态：NTS
        时间部分：YYYYMMDDhhmm分别表示4位年2位月2位日2位时2位分（24小时制北京时间），不足补0。
     */
    /**
     * 类型 ORD/RTD
     */
    private String type;
    /**
     * 文件类型 .tif .png
     */
    private String fileType;
    /**
     * 卫星/无人机 UAV/SAT
     */
    private String uavOrSat;

    /**
     * 流域代码
     */
    private String streamCode;

    /**
     * 点位
     */
    private String pointNo;
    /**
     * 文件时间-年
     */
    private String fileYear;
    /**
     * 文件时间-月
     */
    private String fileMonth;
    /**
     * 文件时间-日
     */
    private String fileDay;
    /**
     * 文件时间-时
     */
    private String fileHour;
    /**
     * 文件时间-分
     */
    private String fileMinute;
    /**
     * 文件创建时间-年
     */
    private String fileCreatYear;
    /**
     * 文件创建时间-月
     */
    private String fileCreatMonth;
    /**
     * 文件创建时间-日
     */
    private String fileCreatDay;
    /**
     * 文件创建时间-时
     */
    private String fileCreatHour;
    /**
     * 文件创建时间-分
     */
    private String fileCreatMinute;
    /**
     * 数据类型代码，如
     *  悬浮物：SPM
     *  叶绿素浓度：CPC
     *  透明度：TSP
     *  总氮：TNG
     *  浊度：TBD
     *  营养状态：NTS
     */
    private String dataCode;

    private String fileName;
    private boolean isValid = false;

    public TifFileInfo(String fileName) {
        this.fileName = fileName;
        init();
    }
    private void init() {
        /*
        （01）CZTL_采集日期_水样数据_生成时间（YYYYMMDD）.xlsx                                                        4_  采样作业数据检验结果数据文件
        （02）PIC_河流代码（2位）_点位（2位）_序号（3位）_照片日期                                                       5_  采样作业现场照片文件
        （03）SAT_ORD_河流代码（2位）_点位（2位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif                       6_ 卫星原始数据文件
        （04）SAT_ORD_RGB_河流代码（2位）_点位（2位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif                   7_ 卫星原始RGB数据文件 7_
        （05）SAT_ORD_IR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif                7_ 卫星原始拼接红外文件 7_
        （06）SAT_ORD_NIR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif               7_ 卫星原始拼接近红外文件 7_
        （07）SAT_RTD_河流代码（2位）_点位（2位）_反演水质参数（2-3位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif     7_ 卫星反演数据文件 7_
        （08）UAV_ORD_DOM_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif               7_ 无人机原始拼接DOM文件 7_
        （09）UAV_ORD_IR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif                7_ 无人机原始拼接红外文件 7_
        （10）UAV_ORD_NIR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif               7_ 无人机原始拼接近红外文件 7_
        （11）UAV_RTD_河流代码（2位）_点位（2位）_反演水质参数（2-3位）_数据日期（YYYYMMDDHH）_生成时间（YYYYMMDDhhmm）.tif   7_ 无人机反演数据文件 7_
         */

        if (StringUtils.isEmpty(fileName)) {
            return;
        }
        String[] fileNameSplit = fileName.split("_");
        //tif不支持前两种数据文件格式
        if (fileNameSplit.length < 6) {
            return;
        }
        if ((fileName.startsWith("SAT_ORD_RGB_") || fileName.startsWith("SAT_ORD_IR_") ||
                fileName.startsWith("SAT_ORD_NIR_") || fileName.startsWith("SAT_RTD_") ||
                fileName.startsWith("UAV_"))&& fileNameSplit.length < 7) {
            return;
        }
        fileType = fileName.substring(fileName.lastIndexOf(".") + 1);

        //UAV代表无人机，SAT代表卫星
        uavOrSat = fileNameSplit[0];
        String dataString = "";
        String createTime = "";
        if (fileName.startsWith("SAT_")) {
            if (fileName.startsWith("SAT_ORD_RGB_") || fileName.startsWith("SAT_ORD_IR_") || fileName.startsWith("SAT_ORD_NIR_")) {
                /*
                    （04）SAT_ORD_RGB_河流代码（2位）_点位（2位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif                   7_ 卫星原始RGB数据文件 7_
                    （05）SAT_ORD_IR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif                7_ 卫星原始拼接红外文件 7_
                    （06）SAT_ORD_NIR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif               7_ 卫星原始拼接近红外文件 7_
                 */
                type = fileNameSplit[1];
                dataCode = fileNameSplit[2];
                streamCode = fileNameSplit[3];
                pointNo = fileNameSplit[4];
                dataString = fileNameSplit[5];
                createTime = fileNameSplit[6];
            }
            else if (fileName.startsWith("SAT_RTD_")) {
                /*
                     （07）SAT_RTD_河流代码（2位）_点位（2位）_反演水质参数（2-3位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif     7_ 卫星反演数据文件 7_
                 */
                type = fileNameSplit[1];
                dataCode = fileNameSplit[4];
                streamCode = fileNameSplit[2];
                pointNo = fileNameSplit[3];
                dataString = fileNameSplit[5];
                createTime = fileNameSplit[6];
            } else {
                //  （03）SAT_ORD_河流代码（2位）_点位（2位）_数据日期（YYYYMMDD）_生成时间（YYYYMMDDhhmm）.tif                       6_ 卫星原始数据文件
                type = fileNameSplit[1];
                dataCode = "ORIGINAL";
                streamCode = fileNameSplit[2];
                pointNo = fileNameSplit[3];
                dataString = fileNameSplit[4];
                createTime = fileNameSplit[5];
            }
        }else if (fileName.startsWith("UAV_")) {
            if (fileName.startsWith("UAV_ORD_DOM_") || fileName.startsWith("UAV_ORD_IR_") || fileName.startsWith("UAV_ORD_NIR_")) {
                /*
                    （08）UAV_ORD_DOM_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif               7_ 无人机原始拼接DOM文件 7_
                    （09）UAV_ORD_IR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif                7_ 无人机原始拼接红外文件 7_
                    （10）UAV_ORD_NIR_河流代码（2位）_点位（2位）_数据日期（YYYYMMDDhhmm）_生成时间（YYYYMMDDhhmm）.tif               7_ 无人机原始拼接近红外文件 7_
                 */
                type = fileNameSplit[1];
                dataCode = fileNameSplit[2];
                streamCode = fileNameSplit[3];
                pointNo = fileNameSplit[4];
                dataString = fileNameSplit[5];
                createTime = fileNameSplit[6];
            }
            else if (fileName.startsWith("UAV_RTD_")) {
                /*
                     （11）UAV_RTD_河流代码（2位）_点位（2位）_反演水质参数（2-3位）_数据日期（YYYYMMDDHH）_生成时间（YYYYMMDDhhmm）.tif   7_ 无人机反演数据文件 7_
                 */
                type = fileNameSplit[1];
                dataCode = fileNameSplit[4];
                streamCode = fileNameSplit[2];
                pointNo = fileNameSplit[3];
                dataString = fileNameSplit[5];
                createTime = fileNameSplit[6];
            }
        }
        if (!StringUtils.isEmpty(dataString) && !StringUtils.isEmpty(createTime)) {
            isValid = true;
            fileYear = dataString.substring(0, 4);
            fileMonth = dataString.substring(4, 6);
            fileDay = dataString.substring(6, 8);
            if (dataString.length() == 12) {
                fileHour = dataString.substring(8, 10);
                fileMinute = dataString.substring(10, 12);
            } else {
                fileHour = "00";
                fileMinute = "00";
            }
            fileCreatYear = createTime.substring(0, 4);
            fileCreatMonth = createTime.substring(4, 6);
            fileCreatDay = createTime.substring(6, 8);
            if (createTime.length() == 12) {
                fileCreatHour = createTime.substring(8, 10);
                fileCreatMinute = createTime.substring(10, 12);
            } else {
                fileCreatHour = "00";
                fileCreatMinute = "00";
            }
        }
    }
}
