package com.cuit.solarenergy.minio;


import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.cuit.solarenergy.utils.AssertUtil;
import io.netty.handler.codec.EncoderException;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.web.multipart.MultipartFile;
import ws.schild.jave.encode.AudioAttributes;
import ws.schild.jave.encode.EncodingAttributes;
import ws.schild.jave.encode.VideoAttributes;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.UUID;

/**
 * @author wzw
 */
@Slf4j
public class CompressUtil {

    /**
     * 压缩File类型图片
     *
     * @param file
     * @return
     */
    public static File compressFilePicture(File file) throws IOException {
        long fileLength = file.length();
        /**
         * 判断是否是图片
         * 判断是否超过了 100K
         */
        AssertUtil.isTrue(!(isPicture(file.getName().replace(".", "")) && (1024 * 1024 * 0.1) <= fileLength), "当前文件格式不是图片！");
        FileInputStream fileInputStream = null;
        try {
            fileInputStream = new FileInputStream(file.getAbsoluteFile());
            // 判断最大最小值
            if ((1024 * 1024 * 0.1) <= fileLength && fileLength <= (1024 * 1024)) {
                Thumbnails.of(fileInputStream).scale(1f).outputQuality(0.3f);
            }
            // 1 - 2M 的
            else if ((1024 * 1024) < fileLength && fileLength <= (1024 * 1024 * 2)) {
                Thumbnails.of(fileInputStream).scale(1f).outputQuality(0.2f);
            }
            // 2M 以上的
            else if ((1024 * 1024 * 2) < fileLength) {
                Thumbnails.of(fileInputStream).scale(1f).outputQuality(0.1f);
            }
        }finally {
            if ( fileInputStream != null) {
                fileInputStream.close();
            }
        }

        return file;
    }

    /**
     * 压缩MultipartFile类型图片
     *
     * @param file
     * @return
     */
    public static String compressFilePicture(MultipartFile file) throws IOException {
        long size = file.getSize();

        // 判断file是否小于0.1M，如果小于返回null
        if ((1024 * 1024 * 0.1) > size) {
            return null;
        }
        String filePath = System.getProperty("user.dir") + File.separator + UUID.randomUUID().toString().replaceAll("-", "") + "_"+file.getOriginalFilename();
        try {
            // 在项目根目录下的 upload 目录中生成临时文件
            String originalFilename = file.getOriginalFilename();
            String s = file.getOriginalFilename().replace(".", ":").split(":")[1];
//        if (!isPicture(file.getOriginalFilename().replace(".",":").split(":")[1])) {
//            return null;
//        }

            File newFile = new File(filePath);

            // 小于 1M 的
            if ((1024 * 1024 * 0.1) <= size && size <= (1024 * 1024)) {
                Thumbnails.of(file.getInputStream()).scale(1f).outputQuality(0.3f).toFile(newFile);
            }
            // 1 - 2M 的
            else if ((1024 * 1024) < size && size <= (1024 * 1024 * 2)) {
                Thumbnails.of(file.getInputStream()).scale(1f).outputQuality(0.2f).toFile(newFile);
            }
            // 2M 以上的
            else if ((1024 * 1024 * 2) < size) {
                Thumbnails.of(file.getInputStream()).scale(1f).outputQuality(0.1f).toFile(newFile);
            }
        }catch (Exception e) {
            log.error("压缩文件"+file.getOriginalFilename()+"时出错：" + e.getMessage());
        }
        return filePath;
    }

    public static boolean isVideo(String filename) {
        String[] videoExtensions = {"mp4", "avi", "mkv", "flv", "mov", "wmv"};
        String extension = filename.substring(filename.lastIndexOf(".") + 1).toLowerCase();
        return Arrays.asList(videoExtensions).contains(extension);
    }

    /**
     * 视频压缩
     * @param source 源文件
     * @param target 目标文件
     * @param rate 压缩比
     */
    public static void compressVideo(File source , File target , Integer rate) throws EncoderException {
        try {
            System.out.println("---------------开始压缩---------------");
            long start = System.currentTimeMillis();

            // 音频编码属性配置
            AudioAttributes audio= new AudioAttributes();
            audio.setCodec("libmp3lame");
            // 设置音频比特率,单位:b (比特率越高，清晰度/音质越好，当然文件也就越大 56000 = 56kb)
            // audio.setBitRate(new Integer(56_000));
            audio.setBitRate(new Integer(10));
            // 设置重新编码的音频流中使用的声道数（1 =单声道，2 = 双声道（立体声））
            audio.setChannels(1);
            // 采样率越高声音的还原度越好，文件越大
            // audio.setSamplingRate(new Integer(44100));
            audio.setSamplingRate(new Integer(22050));
            // 视频编码属性配置
            VideoAttributes video=new VideoAttributes();
            // 设置编码
            video.setCodec("mpeg4");
            //设置音频比特率,单位:b (比特率越高，清晰度/音质越好，当然文件也就越大 5600000 = 5600kb)
            // video.setBitRate(new Integer(5_600_000 / rate));
            video.setBitRate(new Integer(10 / rate));

            // 设置视频帧率（帧率越低，视频会出现断层，越高让人感觉越连续）,这里 除1000是为了单位转换
            video.setFrameRate(new Integer(15));


            // 编码设置
            EncodingAttributes attr=new EncodingAttributes();
            attr.setOutputFormat("mp4");
            attr.setAudioAttributes(audio);
            attr.setVideoAttributes(video);

//            // 设置值编码
//            Encoder ec = new Encoder();
//            ec.encode(new MultimediaObject(source), target, attr);


            System.out.println("---------------结束压缩---------------");
            long end = System.currentTimeMillis();
            System.out.println("压缩前大小："+source.length() + " 压缩后大小：" + target.length());
            System.out.println("压缩耗时：" + (end -start));

        } catch (EncoderException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        }

    }




    /**
     * 判断文件是否为图片
     */
    public static boolean isPicture(String imgName) {
        boolean flag = false;
        if (StringUtils.isBlank(imgName)) {
            return false;
        }
        String[] arr = {"bmp", "dib", "gif", "jfif", "jpe", "jpeg", "jpg", "pdf", "png", "tif", "tiff", "ico", "JPG", "PNG"};
        for (String item : arr) {
            if (item.equals(imgName)) {
                flag = true;
                break;
            }
        }
        return flag;
    }

}
