package com.cuit.solarenergy.minio;


import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.utils.DateUtil;
import com.cuit.solarenergy.entity.*;
import com.cuit.solarenergy.entity.vo.req.PicAviListGetVoReq;
import com.cuit.solarenergy.entity.vo.ret.PageVo;
import com.cuit.solarenergy.exception.ParamsException;
import com.cuit.solarenergy.mapper.*;
import com.cuit.solarenergy.utils.AssertUtil;
import com.cuit.solarenergy.utils.ErrorMessage;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;


@Service
@Slf4j
public class MinioServiceImpl implements MinioService {
    @Autowired
    private TMinioMapper tMinioMapper;

    @Autowired
    private MinioServer minioServer;

    @Autowired
    private UavDataMapper uavDataMapper;

    @Autowired
    private UavInvDataMapper uavInvDataMapper;

    @Autowired
    private UavSendDataMapper uavSendDataMapper;

    @Autowired
    private SatelliteDataMapper satelliteDataMapper;

    @Autowired
    private SatelliteInvDataMapper satelliteInvDataMapper;

    @Autowired
    private SatelliteSendDataMapper satelliteSendDataMapper;

    @Autowired
    private TStreamBaseInfoMapper tStreamBaseInfoMapper;
    @Resource
    private MinioConfig properties;

    /**
     * 以MultipartFile类形式上传文件
     *
     * @param file 文件对象
     * @return AppendixInfo 附件信息类
     */
    @Override
    public MinioInfo uploadFile(MultipartFile file) {
        MinioInfo appendixInfo = new MinioInfo();
        String compressFileUrl = "", fileUrl = "";
        Integer attachmentId = null;
        String fileName = "";
        String type = "";
        String fileType = "";
        //获取文件大小
        String fileSize = formatFileSize(file.getSize());
        try {
            AssertUtil.isTrue(ObjectUtil.isEmpty(file), "传入文件不能为空");
            //新增附件类，填入获取的标识并入库
            fileName = file.getOriginalFilename();
            String[] fileNameSplit = fileName.split("_");
            //PIC代表图片，AVI代表视频
            String picOrAvi = fileNameSplit[0];
            //所属流域
            String streamCode = fileNameSplit[1];
            //minio目录
            String minioDir = fileNameSplit[4].substring(0, 4) + "/" + fileNameSplit[4].substring(4, 6) + "/" + fileNameSplit[4].substring(6, 8);
            //创建时间
            String creatTime = fileNameSplit[4].substring(0, 4) + "-" + fileNameSplit[4].substring(4, 6) + "-" + fileNameSplit[4].substring(6, 8);
            //type = fileName.substring(file.getOriginalFilename().lastIndexOf("."));
            //fileType = type.substring(1);//上传文件获取唯一标识

            String objectName = minioServer.uploadFile(file, minioDir, fileName, true);

            //通过标识获取文件的url和缩略图的url
            fileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectName);
            if (picOrAvi.equals("PIC")) {
                compressFileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectName);
            }
            TMinio tMinio = TMinio.builder().stream(streamCode).createTime(creatTime).fileUrlMin(compressFileUrl).fileName(fileName).fileType(picOrAvi).fileSize(fileSize)
                    .minioId(fileUrl).build();
            tMinioMapper.insert(tMinio);
            //获取入库后的的id
            attachmentId = tMinio.getId();
            log.info("已处理完文件：" + fileName);
        } catch (ParamsException e) {
            // 当参数错误时
            if ("当前路径所指文件在minio中不存在！".equals(e.getMessage())) {
                compressFileUrl = null;
            } else {
                e.printStackTrace();
                throw new ErrorMessage(e.getMessage());
            }
        } catch (Exception e) {
            log.error("上传" + file + "附件信息失败:{}", e.getMessage());
            e.printStackTrace();
            throw new ErrorMessage(e.getMessage());
        } finally {
            //将获取的信息装入AppendixInfo
            appendixInfo = MinioInfo.builder().attachmentId(attachmentId).fileName(fileName).fileSize(fileSize)
                    .fileType(fileType).fileUrl(fileUrl).compressFileUrl(compressFileUrl).build();
        }
        return appendixInfo;
    }

    /**
     * 以MultipartFile类形式上传图片
     *
     * @param file 文件对象
     * @param type
     * @return AppendixInfo 附件信息类
     */
    @Override
    public MinioInfo uploadFile(MultipartFile file, String streamName, String siteName, String type) {
        MinioInfo appendixInfo = new MinioInfo();
        String compressFileUrl = "", fileUrl = "";
        Integer attachmentId = null;
        String fileName = "";
        String fileType = "";
        //获取文件大小
        String fileSize = formatFileSize(file.getSize());
        try {
            AssertUtil.isTrue(ObjectUtil.isEmpty(file), "传入文件不能为空");
            //新增附件类，填入获取的标识并入库
            fileName = file.getOriginalFilename();
            String[] fileNameSplit = fileName.split("_");

            //PIC代表图片，AVI代表视频
//            String picOrAvi = fileNameSplit[0];
            //所属流域
//            String streamCode = fileNameSplit[1];

            LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TStreamBaseInfo::getStreamName, streamName);
            TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(queryWrapper);
            String streamCode = tStreamBaseInfo.getStreamCode();
            //minio目录
            Calendar calendar = Calendar.getInstance();
            // 获取当前年
            int year = calendar.get(Calendar.YEAR);
            // 获取当前月
            int month = calendar.get(Calendar.MONTH) + 1;
            // 获取当前日
            int day = calendar.get(Calendar.DATE);
            String minioDir = year + "/" + month + "/" + day;
            //创建时间
            String creatTime = year + "-" + month + "-" + day;

            //type = fileName.substring(file.getOriginalFilename().lastIndexOf("."));
            //fileType = type.substring(1);//上传文件获取唯一标识

            String objectName = minioServer.uploadFile(file, minioDir, fileName, true);

            String minioYsDir = minioDir;

            String objectCompressName = null;
            //压缩文件
            String sCompressFile = "";

            if (CompressUtil.isPicture(file.getOriginalFilename().replace(".", ":").split(":")[1])) {
                String filenameExtension = StringUtils.getFilenameExtension(fileName);
                String compressFileName = getCompressFileName(fileName, filenameExtension);
                sCompressFile = CompressUtil.compressFilePicture(file);
                if (!StringUtils.isEmpty(sCompressFile)) {
                    File newFile = new File(sCompressFile);
                    objectCompressName = minioServer.uploadFile(newFile, minioYsDir, compressFileName, true);
                }
            }

            if (CompressUtil.isVideo(file.getOriginalFilename().replace(".", ":").split(":")[1])) {
                // 将固定AVI图片写入到filePath
//                String filePath = System.getProperty("user.dir") + File.separator + "data" + File.separator + "avi" + File.separator +"avi.jpg";
                String filePath = File.separator + "data" + File.separator + "avi" + File.separator +"avi.jpg";
                String compressFileName = getCompressFileName(fileName, "jpg");
                File newFile = new File(filePath);
                objectCompressName = minioServer.uploadFile(newFile, minioYsDir, compressFileName, true);
            }

            //通过标识获取文件的url和缩略图的url
            fileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectName);
            compressFileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectCompressName);  //minioServer.getCompressPermanentShareURL(objectName);
            TMinio tMinio = TMinio.builder().stream(streamCode).createTime(creatTime).fileUrlMin(compressFileUrl).fileName(fileName).fileType(type).fileSize(fileSize)
                    .minioId(fileUrl).build();

            tMinioMapper.insert(tMinio);
            //获取入库后的的id
            attachmentId = tMinio.getId();
        } catch (ParamsException e) {
            // 当参数错误时
            if ("当前路径所指文件在minio中不存在！".equals(e.getMessage())) {
                compressFileUrl = null;
            } else {
                e.printStackTrace();
                throw new ErrorMessage(e.getMessage());
            }
        } catch (Exception e) {
            log.error("上传" + file + "附件信息失败:{}", e.getMessage());
            e.printStackTrace();
            throw new ErrorMessage(e.getMessage());
        } finally {
            //将获取的信息装入AppendixInfo
            appendixInfo = MinioInfo.builder().attachmentId(attachmentId).fileName(fileName).fileSize(fileSize)
                    .fileType(fileType).fileUrl(fileUrl).compressFileUrl(compressFileUrl).build();
        }
        return appendixInfo;
    }


    /**
     * 以MultipartFile类形式上传视频
     *
     * @param file 文件对象
     * @return AppendixInfo 附件信息类
     */
    @Override
    public MinioInfo uploadVideo(MultipartFile file, String streamName, String siteName) {
        MinioInfo appendixInfo = new MinioInfo();
        String compressFileUrl = "", fileUrl = "";
        Integer attachmentId = null;
        String fileName = "";
        String type = "";
        String fileType = "";
        //获取文件大小
        String fileSize = formatFileSize(file.getSize());
        try {
            AssertUtil.isTrue(ObjectUtil.isEmpty(file), "传入文件不能为空");
            //新增附件类，填入获取的标识并入库
            fileName = file.getOriginalFilename();
            String[] fileNameSplit = fileName.split("_");
            //PIC代表图片，AVI代表视频
//            String picOrAvi = fileNameSplit[0];
            //所属流域
//            String streamCode = fileNameSplit[1];
            LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TStreamBaseInfo::getStreamName, streamName);
            TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(queryWrapper);
            String streamCode = tStreamBaseInfo.getStreamCode();
            //minio目录
            Calendar calendar = Calendar.getInstance();
            // 获取当前年
            int year = calendar.get(Calendar.YEAR);
            // 获取当前月
            int month = calendar.get(Calendar.MONTH) + 1;
            // 获取当前日
            int day = calendar.get(Calendar.DATE);
            String minioDir = year + "/" + month + "/" + day;
            //创建时间
            String creatTime = year + "-" + month + "-" + day;
            //type = fileName.substring(file.getOriginalFilename().lastIndexOf("."));
            //fileType = type.substring(1);//上传文件获取唯一标识

            String objectName = minioServer.uploadFile(file, minioDir, fileName, true);

            //通过标识获取文件的url和缩略图的url
            fileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectName);
            compressFileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectName);

            TMinio tMinio = TMinio.builder().stream(streamCode).createTime(creatTime).fileUrlMin(compressFileUrl).fileName(fileName).fileType("AVI").fileSize(fileSize)
                    .minioId(fileUrl).build();
            tMinioMapper.insert(tMinio);
            //获取入库后的的id
            attachmentId = tMinio.getId();
        } catch (ParamsException e) {
            // 当参数错误时
            if ("当前路径所指文件在minio中不存在！".equals(e.getMessage())) {
                compressFileUrl = null;
            } else {
                e.printStackTrace();
                throw new ErrorMessage(e.getMessage());
            }
        } catch (Exception e) {
            log.error("上传" + file + "附件信息失败:{}", e.getMessage());
            e.printStackTrace();
            throw new ErrorMessage(e.getMessage());
        } finally {
            //将获取的信息装入AppendixInfo
            appendixInfo = MinioInfo.builder().attachmentId(attachmentId).fileName(fileName).fileSize(fileSize)
                    .fileType(fileType).fileUrl(fileUrl).compressFileUrl(compressFileUrl).build();
        }
        return appendixInfo;
    }

    /**
     * 以MultipartFile类形式上传tif文件
     *
     * @param file     文件对象(原tif文件)
     * @param file     预览图片文件对象
     * @param id       GeoServer发布的id
     * @param bbox     边框
     * @param max      最大值
     * @param min      最小值
     * @param avg      平均值
     * @param bands   波段
     */
    @Override
    public MinioInfo uploadForTif(MultipartFile file, MultipartFile pngfile, Long id, String bbox, Double max, Double min, Double avg, int bands) throws Exception {
        MinioInfo appendixInfo = new MinioInfo();
        String compressFileUrl = "";
        String fileUrl = "";
        String pngFileUrl = "";
        Integer attachmentId = null;
        String fileName = "";
        String type = "";
        String fileType = "";
        //获取文件大小
        String fileSize = formatFileSize(file.getSize());
        try {
            AssertUtil.isTrue(ObjectUtil.isEmpty(file), "传入文件不能为空");
            //新增附件类，填入获取的标识并入库
            fileName = file.getOriginalFilename();
            TifFileInfo tifFileInfo = new TifFileInfo(fileName);
            if (!tifFileInfo.isValid()) {
                throw new ParamsException("文件名格式错误");
            }



            //minio目录
            String minioDir = tifFileInfo.getFileYear() + "/" + tifFileInfo.getFileMonth()+ "/" + tifFileInfo.getFileDay();
            //创建时间
            String creatTime =tifFileInfo.getFileCreatYear() + "-" + tifFileInfo.getFileCreatMonth() + "-" + tifFileInfo.getFileCreatDay();
            Date dateCreatTime = DateUtil.parseDate(creatTime);
            fileType = tifFileInfo.getFileType();
            //数据代码，如CPC等
            String dataCode =  tifFileInfo.getDataCode();

            String objectName = minioServer.uploadFile(file, minioDir, fileName, true);

            //保存在数据库中的Minoid修改为ObjectName  minio对象唯一标识
            ////通过标识获取文件的url

            fileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectName);
            String filenameExtension = StringUtils.getFilenameExtension(pngfile.getOriginalFilename());

            String pngFileName = getCompressFileName(fileName, filenameExtension);

            String objectCompressName = minioServer.uploadFile(pngfile, minioDir, pngFileName, true);

            pngFileUrl = minioServer.getPermanentShareURLWithoutVisitPoint(objectCompressName);

            //保存入数据库
            if (tifFileInfo.getType().equals("RTD")) {
                //所属流域
                String streamCode = tifFileInfo.getStreamCode();
                //水质参数
                String index = tifFileInfo.getDataCode();

                if (tifFileInfo.getUavOrSat().equals("UAV")) {
                    UavInvData uavInvData = UavInvData.builder()
                            .streamCode(streamCode).registerTime(dateCreatTime).indexName(index).originalFileName(fileName).minioId(fileUrl)
                            .maxNumber(max).minNumber(min).avgNumber(avg).geoserverInfoId(Integer.valueOf("" + id)).bbox(bbox).pngMinioId(pngFileUrl)
                            .build();
                    uavInvDataMapper.insert(uavInvData);
                    //获取入库后的的id
                    attachmentId = (Integer) uavInvData.getId();
                }

                if (tifFileInfo.getUavOrSat().equals("SAT")) {
                    SatelliteInvData satelliteInvData = SatelliteInvData.builder()
                            .streamCode(streamCode).registerTime(dateCreatTime).indexName(index).originalFileName(fileName).minioId(fileUrl).satellite("1")
                            .maxNumber(max).minNumber(min).avgNumber(avg).geoserverInfoId(Integer.valueOf("" + id)).bbox(bbox).pngMinioId(pngFileUrl)
                            .build();
                    satelliteInvDataMapper.insert(satelliteInvData);
                    //获取入库后的的id
                    attachmentId = (Integer) satelliteInvData.getId();
                }
            }
            if (tifFileInfo.getType().equals("ORD")) {
                //所属流域
                String streamCode = tifFileInfo.getStreamCode();
                //波段
                String band = tifFileInfo.getDataCode();
                if (tifFileInfo.getUavOrSat().equals("UAV")) {
                    UavData uavData = UavData.builder()
                            .streamCode(streamCode).bbox(bbox).originalFileName(fileName).minioId(fileUrl).registerTime(dateCreatTime).bandNumber(bands).dataCode(dataCode)
                            .build();
                    uavDataMapper.insert(uavData);
                    //获取入库后的的id
                    attachmentId = (Integer) uavData.getId();

                    UavSendData uavSendData = UavSendData.builder()
                            .pid(attachmentId).bandname(band).geoserverInfoId(id).pngMinioId(pngFileUrl)
                            .build();
                    uavSendDataMapper.insert(uavSendData);
                }

                if (tifFileInfo.getUavOrSat().equals("SAT")) {
                    SatelliteData satelliteData = SatelliteData.builder()
                            .streamCode(streamCode).bbox(bbox).originalFileName(fileName).minioId(fileUrl).registerTime(dateCreatTime).satellite("1").bandNumber(bands).dataCode(dataCode)
                            .build();
                    satelliteDataMapper.insert(satelliteData);
                    //获取入库后的的id
                    attachmentId = (Integer) satelliteData.getId();

                    SatelliteSendData satelliteSendData = SatelliteSendData.builder()
                            .pid(attachmentId).bandname(band).geoserverInfoId(id).pngMinioId(pngFileUrl)
                            .build();
                    satelliteSendDataMapper.insert(satelliteSendData);
                }
            }
        } catch (ParamsException e) {
            // 当参数错误时
            if ("当前路径所指文件在minio中不存在！".equals(e.getMessage())) {
                compressFileUrl = null;
            } else {
                e.printStackTrace();
                throw new Exception(e.getMessage());
            }
        } catch (Exception e) {
            log.error("上传" + file + "附件信息失败:{}", e.getMessage());
            e.printStackTrace();
            throw new Exception(e.getMessage());
        } finally {
            //将获取的信息装入AppendixInfo
            appendixInfo = MinioInfo.builder().attachmentId(attachmentId).fileName(fileName).fileSize(fileSize)
                    .fileType(fileType).fileUrl(fileUrl).compressFileUrl(compressFileUrl).build();
        }
        return appendixInfo;
    }

    @NotNull
    private static String getCompressFileName(String fileName, String filenameExtension) {
        //为了方便管理，缩略图为原文件_ys.png命名，并且与原文件放在一个目录中
        String fileNameWithoutExtension = StringUtils.cleanPath(StringUtils.stripFilenameExtension(fileName));

        String compressFileName = "";

        if (!org.apache.commons.lang3.StringUtils.isEmpty(filenameExtension)) {
            compressFileName = fileNameWithoutExtension + "_ys" + "." + filenameExtension;
        } else {
            compressFileName = fileNameWithoutExtension;
        }
        return compressFileName;
    }

    @Override
    public List<MinioInfo> batchUploadFile(MultipartFile[] files) {
        List<MinioInfo> appendixInfos = new ArrayList<>();
        for (MultipartFile multipartFile : files) {
            appendixInfos.add(uploadFile(multipartFile));
        }
        return appendixInfos;
    }

    /**
     * 批量删除附件信息
     *
     * @param attachmentIds 附件ids
     * @return Boolean
     */
    @Override
    public Boolean deleteAppendix(String attachmentIds) {
        String[] attachmentId = attachmentIds.replaceAll("，", ",").split(",");
        int ret = 0;
//        try {
        ArrayList<String> idList = new ArrayList<String>(attachmentId.length);
        Collections.addAll(idList, attachmentId);
        for (String s : idList) {
            //找出对应的附件信息
            TMinio tMinio = tMinioMapper.selectById(s);
            AssertUtil.isTrue(ObjectUtils.isEmpty(tMinio), "无该附件！");
            //根据标识在Minio中删除文件
            if (!ObjectUtils.isEmpty(tMinio.getMinioId())) {
                minioServer.deleteFile(tMinio.getMinioId());
            }
            tMinio.setIsDeleted("1");
            tMinioMapper.updateById(tMinio);
            ret = 1;
        }

//            ret = attachmentMapper.deleteBatchIds(idList) >= 1;
//        } catch (Exception e) {
//            log.error("批量删除附件信息失败！");
//            e.printStackTrace();
//        }
        return (ret == 1);
    }

    @Override
    public JSONObject uploadByStream(String fileName, MultipartFile file) throws IOException {
        MinIoHttpUtil minIoHttpUtil = new MinIoHttpUtil();
        minIoHttpUtil.setUrl(properties.getEndpoint());

        JSONObject object = minIoHttpUtil.uploadFile("/uploadByStream", properties.getBucketName(), fileName, file);
        return object;
    }

    @Override
    public JSONObject getFileUrl(String fileName, Integer expires) {
        MinIoHttpUtil minIoHttpUtil = new MinIoHttpUtil();
        minIoHttpUtil.setUrl(properties.getEndpoint());
        String https = minIoHttpUtil.getHttps("/getFileUrl?bucketName=" + properties.getBucketName() + "&fileName=" + fileName + "&expires=" + expires);
        JSONObject object = JSONObject.parseObject(https);
        return object;
    }

    @Override
    public void downloadFile(String objectName, HttpServletResponse response) throws BcException {
        minioServer.downloadFile(objectName, response);
    }

    @Override
    public PageVo<TMinio> getPicAviList(PicAviListGetVoReq req) {
        String streamCode = null;
        Page<TMinio> page = new Page<>(req.getCurrent(), req.getSize());
        if (req.getStreamName() != null) {
            LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TStreamBaseInfo::getStreamName, req.getStreamName());
            TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(queryWrapper);
            if (tStreamBaseInfo != null) {
                streamCode = tStreamBaseInfo.getStreamCode();
            }
        }
        page = tMinioMapper.getList(page, streamCode, req.getFileType(), req.getKeywords());
        //对Minio地址进行拼接
        if (page != null) {
            for (int i = 0; i < page.getRecords().size(); i++) {
                page.getRecords().get(i).setFileUrl(getMinioUrl(page.getRecords().get(i).getFileUrl()));
                page.getRecords().get(i).setFileUrlMin(getMinioUrl(page.getRecords().get(i).getFileUrlMin()));
            }
            return new PageVo<>(page);
        }

        return new PageVo<>(page);
    }

    /**
     * 批量上传图片和视频
     *
     * @param files
     * @return
     */
    @Override
    public List<MinioInfo> uploadFiles(MultipartFile[] files) {
        List<MinioInfo> minioInfos = new ArrayList<>();
        for (int i = 0; i < files.length; i++) {
            MinioInfo appendixInfo = new MinioInfo();
            if (!files[i].getOriginalFilename().startsWith("PIC_") && !files[i].getOriginalFilename().startsWith("AVI_") || files[i].getOriginalFilename().split("_").length != 5) {
                log.info("文件:" + files[i].getOriginalFilename() + "上传失败！ --- 原因：" + "命名不符合规范！");
                appendixInfo.setFileName("文件:" + files[i].getOriginalFilename() + "上传失败！ --- 原因：" + "命名不符合规范！");
                minioInfos.add(appendixInfo);
                continue;
            }
            String streamCode = files[i].getOriginalFilename().split("_")[1];
            LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TStreamBaseInfo::getStreamCode, streamCode);
            TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(queryWrapper);
            String streamName = "";
            if (tStreamBaseInfo != null) {
                streamName = tStreamBaseInfo.getStreamName();
            }else {
                log.info("文件:" + files[i].getOriginalFilename() + "上传失败！ --- 原因：" + "流域代码错误！！");
                appendixInfo.setFileName("文件:" + files[i].getOriginalFilename() + "上传失败！ --- 原因：" + "流域代码错误！！");
                minioInfos.add(appendixInfo);
                continue;
            }
            String type = files[i].getOriginalFilename().split("_")[0];
            appendixInfo = uploadFile(files[i], streamName, null, type);
            minioInfos.add(appendixInfo);
        }
        return minioInfos;
    }

    /**
     * 格式化文件大小
     *
     * @param fileLength 单位b
     * @return String
     */
    public static String formatFileSize(Long fileLength) {
        String fileSizeString = "";
        if (fileLength == null) {
            return fileSizeString;
        }
        DecimalFormat df = new DecimalFormat("#.00");
        if (fileLength < 1024) {
            fileSizeString = df.format((double) fileLength) + "B";
        } else if (fileLength < 1048576) {
            fileSizeString = df.format((double) fileLength / 1024) + "K";
        } else if (fileLength < 1073741824) {
            fileSizeString = df.format((double) fileLength / 1048576) + "M";
        } else {
            fileSizeString = df.format((double) fileLength / 1073741824) + "G";
        }
        return fileSizeString;
    }

    private String getMinioUrl(String minioID) {
        String url = properties.getVisitPoint();
        if (org.apache.commons.lang3.StringUtils.isEmpty(minioID)) {
            return minioID;
        }
        if (!(url.endsWith(File.separator) && url.endsWith(File.pathSeparator))) {
            url = url + File.separator;
        }
        String miniourl = url + minioID;
        return miniourl;
    }
}
