package com.cuit.solarenergy.service;

import com.cuit.solarenergy.entity.TSiteBaseInfo;
import com.cuit.solarenergy.entity.vo.req.ChartBaseInfoVoReq;
import com.cuit.solarenergy.entity.vo.req.SiteListGetVoReq;
import com.cuit.solarenergy.entity.vo.ret.*;

import java.util.List;

/**
 * @author PYJ
 * @description 针对表【t_site_base_info(站点基本信息表)】的数据库操作Service
 * @createDate 2023-06-08 11:37:12
 */
public interface TSiteBaseInfoService {
    /**
     * @Description: 统计分析模块根据经纬度查询最近的三个站点
     * @Param: Double lon, Double lat
     * @return: List<TSiteBaseInfo>
     * @Author: LuoSir
     * @Date: 2023/7/17
     */

    List<DataBaseSeparation> getList(Double lon, Double lat);

//
//    /**
//     * @Description: 根据站点、水质参数获取曲线所需的列表
//     * @Param:
//     * @return:
//     * @Author: LuoSir
//     * @Date: 2023/8/7
//     */
//
//    List<StatisticQueryVoRet> queryDataCurveByParams(StatisticQueryVoReq statisticQueryVoReq);


    /**
     *方法功能：返回所有站点经纬度
     *
     * @param
     * @date 2023/8/6 16:36
     * @author Tjb
     * @return
     */

    List<SiteBaseInfoVoRet> getAllSiteList() throws Exception;

    /**
     *
     * 返回绘制图表的所有信息
     */
    List<StatisticAnalysisVoRet> getAllChartList(ChartBaseInfoVoReq chartBaseInfoVoReq) throws Exception;


    PageVo<TSiteBaseInfo> getSiteList(SiteListGetVoReq req);

    List<SiteAndValueVoRet> getValueOfSite(String streamName, String waterParameter);

    List<YearAndCountVoRet> getAllInfoForThreeYear(String streamName);

    List<MonthAndCountVoRet> getCountOfUAV(String streamName);

    List<MonthAndCountVoRet> getCountOfSAT(String streamName);
}
