package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.solarenergy.entity.TDictData;
import com.cuit.solarenergy.entity.vo.ret.Stream;
import com.cuit.solarenergy.mapper.*;
import com.cuit.solarenergy.service.CommonService;
import com.cuit.solarenergy.utils.AssertUtil;
import com.cuit.solarenergy.vo.TDataOriginMethod;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-06-17  09:52
 * @Description: 检测数据实现类
 * @Version: 1.0
 */
@Service
@Slf4j
public class CommonServiceImpl implements CommonService {

    @Autowired
    private TStreamBaseInfoMapper tStreamBaseInfoMapper;

    @Autowired
    private TSamplingIndexMapper tSamplingIndexMapper;

    @Autowired
    private TDataOriginMethodMapper tDataOriginMethodMapper;

    @Autowired
    private TSamplingSiteImageMapper tSamplingSiteImageMapper;

    @Autowired
    private TSiteBaseInfoMapper tSiteBaseInfoMapper;

    @Autowired
    private TDictDataMapper tDictDataMapper;

    @Autowired
    private StreamBboxMapper streamBboxMapper;

    /**
     * 方法功能：获取所有流域名称
     *
     * @param
     * @return List<String>
     * @date 2023/6/17 9:53
     * @author Tjb
     */
    @Override
    public List<Stream> getAllStreamName() {
        List<Stream> allStreamName = tStreamBaseInfoMapper.getAllStreamName();
        log.info("获取到： {} 流域名称", allStreamName);
        return allStreamName;
    }

    /**
     * 方法功能：获取所有反演数据(水质参数)
     *
     * @param
     * @return
     * @date 2023/6/24 19:05
     * @author Tjb
     */
    @Override
    public List<String> getAllWaterQualityParameters() {
        List<String> allWaterQualityParameters = tSamplingIndexMapper.getAllWaterQualityParameters();
        log.info("获取到： {}  反演参数", allWaterQualityParameters);
        return allWaterQualityParameters;
    }

    /**
     * 方法功能：查询所有的数据获取方式
     *
     * @param
     * @return
     * @date 2023/6/24 19:20
     * @author Tjb
     */

    @Override
    public List<TDataOriginMethod> getAllDataOriginal() {
        List<TDataOriginMethod> allDataOriginal = tDataOriginMethodMapper.getAllDataOriginal();
        log.info("查询所有的数据获取方式:  {} ", allDataOriginal);
        return allDataOriginal;
    }

    /**
     * 方法功能：获取原始影像数据集的集合
     *
     * @param
     * @return
     * @date 2023/7/4 9:56
     * @author Tjb
     */

    @Override
    public List<String> getOriginalDataSet() {
        List<String> originalDataSet = tDictDataMapper.getOriginalDataSet();
        log.info("获取原始影像数据集的集合:  {} ", originalDataSet);
        return originalDataSet;
    }


    /**
     * 方法功能：通过缩略图查询真实图片URL
     *
     * @param simplePicture
     * @return
     * @date 2023/7/3 15:15
     * @author Tjb
     */

    @Override
    public String getRealPictureForOne(String simplePicture) {
        log.info("图片缩略图URL:  {} ", simplePicture);
        AssertUtil.isTrue(simplePicture.isEmpty(), "图片缩略图URL为空！！！");
        String realPictureForOne = tSamplingSiteImageMapper.getRealPictureForOne(simplePicture);
        log.info("通过缩略图查询真实图片URL:  {} ", realPictureForOne);
        return realPictureForOne;
    }

    /**
     * 方法功能：通过缩略图查询真实无人机URL
     *
     * @param simpleUAV
     * @return
     * @date 2023/7/4 10:43
     * @author Tjb
     */

    @Override
    public String getRealUAVForOne(String simpleUAV) {
        log.info("无人机缩略图URL:  {} ", simpleUAV);
        AssertUtil.isTrue(simpleUAV.isEmpty(), "无人机缩略图URL为空！！！");
        String realUAVForOne = tSamplingSiteImageMapper.getRealUAVForOne(simpleUAV);
        log.info("通过缩略图查询真实无人机URL:  {} ", realUAVForOne);
        return realUAVForOne;
    }

    /**
     * 方法功能：获取卫星列表
     *
     * @param
     * @return
     * @date 2023/8/9 19:26
     * @author Tjb
     */

    @Override
    public List<Object> getSatelliteList() {
        LambdaQueryWrapper<TDictData> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select(TDictData::getLabel)
                .eq(TDictData::getDictType, "satellite")
                .orderByAsc(TDictData::getSort);
        List<Object> list = tDictDataMapper.selectObjs(queryWrapper);
        return list;
    }

    /**
     *方法功能：获取流域bbox
     *
     * @param streamCodes
     * @date 2023/8/9 20:06
     * @author Tjb
     * @return
     */

    @Override
    public List<String> getStreamBbox(List<String> streamCodes) {
        List<String> list = new ArrayList<>();
        list = streamBboxMapper.getStreamBbox(streamCodes);
        return list;
    }

    @Override
    public List<String> getAllStreamCode() {
        List<String> allStreamCode = tStreamBaseInfoMapper.getAllStreamCode();
        log.info("获取到： {} 流域代码", allStreamCode);
        return allStreamCode;
    }

    @Override
    public List<String> getAllSiteLocation() {
        return tDictDataMapper.getAllSiteLocation();

    }

    @Override
    public List<Double> getAllSiteDistance() {
        return tDictDataMapper.getAllSiteDistance();
    }

    /**
     * 方法功能：通过缩略图查询真实视频URL
     *
     * @param simpleVideo
     * @return
     * @date 2023/7/3 15:17
     * @author Tjb
     */
    @Override
    public String getRealVideoForOne(String simpleVideo) {
        log.info("视频缩略图URL:  {} ", simpleVideo);
        AssertUtil.isTrue(simpleVideo.isEmpty(), "视频缩略图URL为空！！！");
        String realVideoForOne = tSamplingSiteImageMapper.getRealVideoForOne(simpleVideo);
        return realVideoForOne;
    }

    /**
     * 方法功能：获取某一流域下所有的站点名称
     *
     * @param streamName
     * @return
     * @date 2023/7/3 15:59
     * @author Tjb
     */

    @Override
    public List<String> getStationsForOneStream(String streamName) {
        log.info("获取  {}  流域下所有的站点名称", streamName);
        List<String> stationsForOneStream = tSiteBaseInfoMapper.getStationsForOneStream(streamName);
        log.info("获取到  {}  流域下所有的站点名称为： {} ", streamName, stationsForOneStream);
        return stationsForOneStream;
    }

    /**
     * 方法功能：获取某个站点所属流域名称
     *
     * @param siteId
     * @return
     * @date 2023/7/3 16:09
     * @author Tjb
     */

    @Override
    public String getStationForOneSite(String siteId) {
        log.info("获取 {} 站点所属流域名称", siteId);
        String stationForOneSite = tStreamBaseInfoMapper.getStationForOneSite(siteId);
        log.info("获取 {} 站点所属流域名称为： {} ", siteId, stationForOneSite);
        return stationForOneSite;
    }

}
