package com.cuit.solarenergy.service.impl;

import com.cuit.gis.exception.GeoTiffException;
import com.cuit.gis.manager.MeteGis;
import com.cuit.gis.projections.CoordinateUtil;
import com.cuit.gis.projections.ProjectTrans;
import com.cuit.gis.raster.GeoTiff;
import com.cuit.mete.geoserver.GeoServerListConfig;
import com.cuit.mete.geoserver.GeoServerPublishUtil;
import com.cuit.mete.geoserver.LayerSRSbbox;
import com.cuit.mete.geoserver.geoserverexecption.CoverageStoreNotFoundException;
import com.cuit.mete.geoserver.geoserverexecption.LayerNotFoundException;
import com.cuit.mete.geoserver.geoserverexecption.WMMQueryParameterExeception;
import com.cuit.mete.geoserver.geoserverexecption.WorkSpaceNotFoundException;
import com.cuit.mete.geoserver.improve.GeoServerManager;
import com.cuit.mete.geoserver.improve.GeoServerReader;
import com.cuit.mete.geoserver.wms.RasterpixelGrayIndex;
import com.cuit.mete.geoserver.wms.WMSGetFeatureInfo;
import com.cuit.solarenergy.config.MyGeoServerConfig;
import com.cuit.solarenergy.entity.Geoserverinfo;
import com.cuit.solarenergy.entity.GeoserverinfoBase;
import com.cuit.solarenergy.service.GeoServerPubService;
import com.cuit.solarenergy.service.GeoserverinfoBaseService;
import com.cuit.solarenergy.service.GeoserverinfoService;
import com.cuit.solarenergy.service.SequenceService;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;
import it.geosolutions.geoserver.rest.encoder.GSResourceEncoder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.geotools.geometry.Envelope2D;
import org.locationtech.jts.geom.Point;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.net.MalformedURLException;
import java.util.*;

/**
 * geoServer地图发布相关
 * @author cuitg -- ZengwuWang
 * @date 2023年07月25日 16:32
 */
@Service
@Slf4j
public class GeoServerPubServiceImpl implements GeoServerPubService {
    @Autowired
    MyGeoServerConfig myGeoServerConfig;

    @Resource(type= SequenceServiceGeoServerIDImpl.class)
    SequenceService sequenceService;

    @Autowired
    GeoserverinfoService geoserverinfoService;

    @Autowired
    GeoserverinfoBaseService geoserverinfoBaseService;

    @Override
    public Long pubGeoServerAndSavetoDB(String filePathName) {
        String layer = FilenameUtils.getBaseName(filePathName);

        boolean pubFlag = pubGeoServer(layer, filePathName);
        if (pubFlag) {
             return save2DB(filePathName);
        }
        return -1L;
    }

    @Override
    public Long pubGeoServerAndSavetoDB(String defstyles, String filePathName) {
        String layer = FilenameUtils.getBaseName(filePathName);
        String src;
        GeoTiff geoTiff = null;
        try {
            geoTiff = new GeoTiff(filePathName);
            CoordinateReferenceSystem coordinateReferenceSystem = geoTiff.getCoverage().getCoordinateReferenceSystem();
            String firstEPSG = CoordinateUtil.getFirstEPSG(coordinateReferenceSystem);

            boolean pubFlag = pubGeoServer(layer, firstEPSG, defstyles, filePathName);
            if (pubFlag) {
                return save2DB(filePathName, null, defstyles);
            }
        } catch (GeoTiffException e) {
            log.error("发布GeoServer时出错！" + e.getMessage());
            return -1L;
        }
        return -1L;
    }


    @Override
    public Long pubGeoServerAndSavetoDB(String defstyles, String srs, String filePathName) {
        String layer = FilenameUtils.getBaseName(filePathName);
//        String src;
//        GeoTiff geoTiff = null;
//        try {
//            geoTiff = new GeoTiff(filePathName);
//            CoordinateReferenceSystem coordinateReferenceSystem = geoTiff.getCoverage().getCoordinateReferenceSystem();
//            String firstEPSG = CoordinateUtil.getFirstEPSG(coordinateReferenceSystem);

        boolean pubFlag = pubGeoServer(layer, srs, defstyles, filePathName);
        if (pubFlag) {
            return save2DB(filePathName, srs, defstyles);
        }
//        } catch (GeoTiffException e) {
//            log.error("发布GeoServer时出错！" + e.getMessage());
//            return -1L;
//        }
        return -1L;
    }

    @Override
    public String getRasterWmsUrl(Long pubinfoID) {
        //pubGeoServerConfig
        //http://121.37.165.19:8082/geoserver/cztl/wms?service=WMS&version=1.1.0&request=GetMap&layers=cztl:SAT_RTD_JS_00_CPC_20230405_202304250000&bbox=494883.5,3398883.5,496536.5,3400386.5&width=768&height=698&srs=EPSG:32647&styles=cztl:style_cpc&format=application/openlayers
        Geoserverinfo geoserverinfo = geoserverinfoService.getById(pubinfoID);
        String url = "";
        if (geoserverinfo != null) {

            String geoServerURL = "";
//            if (!StringUtils.isEmpty(geoserverinfo.getUrl())) {
//                geoServerURL = geoserverinfo.getUrl();
//            } else {
                //geoServerURL = pubGeoServerConfig.getGeoserverurl();
            geoServerURL = myGeoServerConfig.getGeoserverapiurl();
//            }
            String workspaces = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverworkspaces())) {
                workspaces= geoserverinfo.getGeoserverworkspaces();
            } else {
                workspaces= myGeoServerConfig.getWorkspaces();
            }
            String layer = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverlayers())) {
                layer= geoserverinfo.getGeoserverlayers();
            }
            String styles = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverstyles())) {
                styles= geoserverinfo.getGeoserverstyles();
            }
            String srs = "";
            if (!StringUtils.isEmpty(geoserverinfo.getSrs())) {
                srs= geoserverinfo.getSrs();
            }
            String bbox = "";
            if (!StringUtils.isEmpty(geoserverinfo.getBbox())) {
                bbox= geoserverinfo.getBbox();
            }
            if (StringUtils.isEmpty(geoServerURL) || StringUtils.isEmpty(layer)  || StringUtils.isEmpty(workspaces)) {
                return null;
            }
            //http://121.37.165.19:8082/geoserver/cztl/wms?service=WMS&version=1.1.0&request=GetMap&layers=cztl:SAT_RTD_JS_00_CPC_20230405_202304250000&bbox=494883.5,3398883.5,496536.5,3400386.5&width=768&height=698&srs=EPSG:32647&styles=cztl:style_cpc&format=application/openlayers
            if (geoServerURL.endsWith("/")) {
                geoServerURL = geoServerURL.substring(0, geoServerURL.length()-1);
            }
//            url = geoServerURL + "/" + workspaces + "/wms?service=WMS&version=1.1.0&request=GetMap&layers=" + workspaces + ":" + layer + "&bbox=" + bbox + "&width=768&height=698&srs=" + srs;
            url = geoServerURL + "/" + workspaces + "/wms?service=WMS&version=1.1.0&request=GetMap&layers=" + workspaces + ":" + layer + "&width=768&height=698&srs=" + srs;
            if (!StringUtils.isEmpty(styles)) {
                url += "&styles=" + workspaces + ":" + styles;
            }
            url += "&FORMAT=image/png&TRANSPARENT=true";
            return url;
        } else {
            return null;
        }
    }

    @Override
    public String getMapBox3857RasterWmsUrl(Long pubinfoID) {
        //pubGeoServerConfig
        //http://121.37.165.19:8082/geoserver/cztl/wms?service=WMS&version=1.1.0&request=GetMap&layers=cztl:SAT_RTD_JS_00_CPC_20230405_202304250000&bbox=494883.5,3398883.5,496536.5,3400386.5&width=768&height=698&srs=EPSG:32647&styles=cztl:style_cpc&format=application/openlayers
        Geoserverinfo geoserverinfo = geoserverinfoService.getById(pubinfoID);
        String url = "";
        if (geoserverinfo != null) {

            String geoServerURL = "";
//            if (!StringUtils.isEmpty(geoserverinfo.getUrl())) {
//                geoServerURL = geoserverinfo.getUrl();
//            } else {
                //geoServerURL = pubGeoServerConfig.getGeoserverurl();
                geoServerURL = myGeoServerConfig.getGeoserverapiurl();
//            }
            String workspaces = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverworkspaces())) {
                workspaces= geoserverinfo.getGeoserverworkspaces();
            } else {
                workspaces= myGeoServerConfig.getWorkspaces();
            }
            String layer = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverlayers())) {
                layer= geoserverinfo.getGeoserverlayers();
            }
            String styles = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverstyles())) {
                styles= geoserverinfo.getGeoserverstyles();
            }
            String srs = "EPSG:3857";

            if (StringUtils.isEmpty(geoServerURL) || StringUtils.isEmpty(layer)  || StringUtils.isEmpty(workspaces)) {
                return null;
            }
            if (geoServerURL.endsWith("/")) {
                geoServerURL = geoServerURL.substring(0, geoServerURL.length()-1);
            }
            //http://121.37.165.19:8082/geoserver/cztl/wms?service=WMS&version=1.1.0&request=GetMap&layers=cztl:SAT_RTD_JS_00_CPC_20230405_202304250000&bbox=494883.5,3398883.5,496536.5,3400386.5&width=768&height=698&srs=EPSG:32647&styles=cztl:style_cpc&format=application/openlayers
//            url = geoServerURL + "/" + workspaces + "/wms?service=WMS&version=1.1.0&request=GetMap&layers=" + workspaces + ":" + layer + "&width=768&height=698&srs=" + srs;
            url = geoServerURL + "/" + workspaces + "/wms?service=WMS&version=1.1.0&request=GetMap&layers=" + workspaces + ":" + layer + "&width=768&height=698&srs=" + srs;
            if (!StringUtils.isEmpty(styles)) {
                url += "&styles=" + workspaces + ":" + styles;
            }
            url += "&FORMAT=image/png&TRANSPARENT=true";
            return url;
        } else {
            return null;
        }
    }
    @Override
    public String getMapBox3857RasterWmsUrlByBaseInfo(Long pubinfoID) {
        //pubGeoServerConfig
        //http://121.37.165.19:8082/geoserver/cztl/wms?service=WMS&version=1.1.0&request=GetMap&layers=cztl:SAT_RTD_JS_00_CPC_20230405_202304250000&bbox=494883.5,3398883.5,496536.5,3400386.5&width=768&height=698&srs=EPSG:32647&styles=cztl:style_cpc&format=application/openlayers
        GeoserverinfoBase geoserverinfo = geoserverinfoBaseService.getById(pubinfoID);
        String url = "";
        if (geoserverinfo != null) {

            String geoServerURL = "";
//            if (!StringUtils.isEmpty(geoserverinfo.getUrl())) {
//                geoServerURL = geoserverinfo.getUrl();
//            } else {
            //geoServerURL = pubGeoServerConfig.getGeoserverurl();
            geoServerURL = myGeoServerConfig.getGeoserverapiurl();
//            }
            String workspaces = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverworkspaces())) {
                workspaces= geoserverinfo.getGeoserverworkspaces();
            } else {
                workspaces= myGeoServerConfig.getWorkspaces();
            }
            String layer = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverlayers())) {
                layer= geoserverinfo.getGeoserverlayers();
            }
            String styles = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverstyles())) {
                styles= geoserverinfo.getGeoserverstyles();
            }
            String srs = "EPSG:3857";

            if (StringUtils.isEmpty(geoServerURL) || StringUtils.isEmpty(layer)  || StringUtils.isEmpty(workspaces)) {
                return null;
            }
            if (geoServerURL.endsWith("/")) {
                geoServerURL = geoServerURL.substring(0, geoServerURL.length()-1);
            }
            //http://121.37.165.19:8082/geoserver/cztl/wms?service=WMS&version=1.1.0&request=GetMap&layers=cztl:SAT_RTD_JS_00_CPC_20230405_202304250000&bbox=494883.5,3398883.5,496536.5,3400386.5&width=768&height=698&srs=EPSG:32647&styles=cztl:style_cpc&format=application/openlayers
//            url = geoServerURL + "/" + workspaces + "/wms?service=WMS&version=1.1.0&request=GetMap&layers=" + workspaces + ":" + layer + "&width=768&height=698&srs=" + srs;
            url = geoServerURL + "/" + workspaces + "/wms?service=WMS&version=1.1.0&request=GetMap&layers=" + workspaces + ":" + layer + "&width=768&height=698&srs=" + srs;
            if (!StringUtils.isEmpty(styles)) {
                url += "&styles=" + workspaces + ":" + styles;
            }
            url += "&FORMAT=image/png&TRANSPARENT=true";
            return url;
        } else {
            return null;
        }
    }


    @Override
    public Map<Long, Double> getRasterValue(List<Long> geoinfoids, double x, double y) {
        return getRasterValue(geoinfoids, "EPSG:4326", x, y);
    }

    @Override
    public Map<Long, Double> getRasterValue(List<Long> geoinfoids, String srs, double x, double y) {

        String wmsServerURL = myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();
        String workspace = myGeoServerConfig.getWorkspaces();

        try {
            List<Geoserverinfo> geoserverinfos = geoserverinfoService.listByIds(geoinfoids);
            if (!CollectionUtils.isEmpty(geoserverinfos)) {
                Map<Long, Double> valueMap = new TreeMap<>();

                WMSGetFeatureInfo wmsGetFeatureInfo = new WMSGetFeatureInfo(wmsServerURL, username, password, workspace);
                String layers = "";
                for (int i = 0; i < geoserverinfos.size(); i++) {
                    layers += geoserverinfos.get(i).getGeoserverworkspaces() + ":" + geoserverinfos.get(i).getGeoserverlayers() + ",";
                }
                if (layers.endsWith(",")) {
                    layers = layers.substring(0, layers.length() - 1);
                }
                double bboxWidth = 2.0;
                double bboxHeight = 2.0;
                //地理坐标系
                if (CoordinateUtil.isGeographicCRS(srs)) {
                    //相关于0.0002度  0.00002 * 111 km = 0.00002 * 111000 m  = 2.22米
                    bboxWidth = 0.00002;
                    bboxHeight = 0.0002;
                }

                List<RasterpixelGrayIndex> mulRasterpixelGrayIndex = wmsGetFeatureInfo.getMulRasterpixelGrayIndex(layers, srs, x, y, bboxWidth, bboxHeight);

                List<Double> values = new ArrayList<>();
                for (int i = 0; i < mulRasterpixelGrayIndex.size(); i++) {
                    String layperNam = mulRasterpixelGrayIndex.get(i).getLayperNam();
                    Long key = null;
                    boolean foundFlag = false;
                    for (int j = 0; j < geoserverinfos.size(); j++) {
                        if (layperNam.equalsIgnoreCase(geoserverinfos.get(j).getGeoserverworkspaces() + ":" + geoserverinfos.get(j).getGeoserverlayers())) {
                            key= geoserverinfos.get(j).getId();
                            foundFlag = true;
                            break;
                        }
                    }
                    if (foundFlag) {
                        valueMap.put(key, mulRasterpixelGrayIndex.get(i).getGrayIndex());
                    }
                }
                return valueMap;
            }
        } catch (MalformedURLException e) {
            log.error("取geoserver图层数据时出错：" + e.getMessage());
        } catch (WMMQueryParameterExeception e) {
            log.error("取geoserver图层数据时出错：" + e.getMessage());
        } catch (FactoryException e) {
            log.error("取geoserver图层数据时出错（坐标系？）：" + e.getMessage());;
        }
        return null;
    }

    @Override
    public boolean getPng(Long geoinfoid, int w, int h, String outPngFileName){
        boolean retFlag = false;
        Geoserverinfo geoserverinfo = geoserverinfoService.getById(geoinfoid);

        if (geoserverinfo != null) {
            GeoServerListConfig  geoServerConfig = new GeoServerListConfig();

            String geoServerURL = "";
//            if (!StringUtils.isEmpty(geoserverinfo.getUrl())) {
//                geoServerURL = geoserverinfo.getUrl();
//            } else {
                geoServerURL = myGeoServerConfig.getGeoserverapiurl();
//            }
            geoServerConfig.setGeoserverurl(geoServerURL);

            String workspaces = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverworkspaces())) {
                workspaces = geoserverinfo.getGeoserverworkspaces();
            } else {
                workspaces = myGeoServerConfig.getWorkspaces();
            }
            geoServerConfig.setWorkspaces(workspaces);

            String layer = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverlayers())) {
                layer = geoserverinfo.getGeoserverlayers();
            }
            List<String> layerList = new ArrayList<>();
            layerList.add(layer);
            geoServerConfig.setLayerList(layerList);

            String styles = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverstyles())) {
                styles = geoserverinfo.getGeoserverstyles();
            }
            List<String> stylesList = new ArrayList<>();
            stylesList.add(styles);
            geoServerConfig.setStylesList(stylesList);

            String bbox = "";
            if (!StringUtils.isEmpty(geoserverinfo.getBbox())) {
                bbox = geoserverinfo.getBbox();
            }
            String srs = "";
            if (!StringUtils.isEmpty(geoserverinfo.getSrs())) {
                srs = geoserverinfo.getSrs();
            }
            if (StringUtils.isEmpty(srs)) {
                srs = "EPSG:4326";
            }
            LayerSRSbbox layerSRSbbox = new LayerSRSbbox();
            layerSRSbbox.setSrs(srs);
            if (!StringUtils.isEmpty(bbox)) {
                String[] split = bbox.split(",");
                if (split.length == 4) {
                    layerSRSbbox.setMinx(Double.parseDouble(split[0]));
                    layerSRSbbox.setMiny(Double.parseDouble(split[1]));
                    layerSRSbbox.setMaxx(Double.parseDouble(split[2]));
                    layerSRSbbox.setMaxy(Double.parseDouble(split[3]));
                    geoServerConfig.setLayerSRSbbox(layerSRSbbox);

                    //根据图片大小调节w h
                    //原始图层的比例
                    double bl1 = (layerSRSbbox.getMaxy() - layerSRSbbox.getMiny()) /(layerSRSbbox.getMaxx() - layerSRSbbox.getMinx());
                    double bl2 = 1.0* h / w;
                    if (bl1 > bl2) {
                        w = (int) Math.round( h / bl1 );
                    } else {
                       h = (int) Math.round( w * bl1 );
                    }
                }
            }

            geoServerConfig.setUsername(myGeoServerConfig.getUsername());
            geoServerConfig.setPassword(myGeoServerConfig.getPassword());
            geoServerConfig.setWidth(w);
            geoServerConfig.setHeight(h);
            return GeoServerPublishUtil.getImgFromUrl(geoServerConfig, outPngFileName,  "png");
        }
        return retFlag;
    }

    @Override
    public String getLatLonBoundingBox(Long geoinfoid) {
        boolean retFlag = false;
        Geoserverinfo geoserverinfo = geoserverinfoService.getById(geoinfoid);
        String bbox= null;

        if (geoserverinfo != null) {
            GeoServerListConfig  geoServerConfig = new GeoServerListConfig();

            String geoServerURL = "";
//            if (!StringUtils.isEmpty(geoserverinfo.getUrl())) {
//                geoServerURL = geoserverinfo.getUrl();
//            } else {
            geoServerURL = myGeoServerConfig.getGeoserverapiurl();
//            }

            String workspaces = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverworkspaces())) {
                workspaces = geoserverinfo.getGeoserverworkspaces();
            } else {
                workspaces = myGeoServerConfig.getWorkspaces();
            }


            String layer = "";
            if (!StringUtils.isEmpty(geoserverinfo.getGeoserverlayers())) {
                layer = geoserverinfo.getGeoserverlayers();
            }
            if (StringUtils.isEmpty(geoServerURL) || StringUtils.isEmpty(layer)  || StringUtils.isEmpty(workspaces)) {
                return null;
            }
            LayerSRSbbox coverageLatLonBoundingBox = GeoServerPublishUtil.getCoverageLatLonBoundingBox(geoServerURL, myGeoServerConfig.getUsername(), myGeoServerConfig.getPassword(), workspaces, layer);

            if (coverageLatLonBoundingBox != null) {
                bbox = coverageLatLonBoundingBox.getMinx() + ","+coverageLatLonBoundingBox.getMiny()+","+coverageLatLonBoundingBox.getMaxx() + ","+coverageLatLonBoundingBox.getMaxy();
            }
        }
        return bbox;
    }

    @Override
    public Boolean removeLayer(String workspace) {
        boolean retFlag = true;
        String wmsServerURL = myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();

        try {
            GeoServerManager geoServerManager = new GeoServerManager(wmsServerURL, username, password);
            GeoServerReader geoServerReader = new GeoServerReader(wmsServerURL, username, password);

            try {
                geoServerManager.removeWorkspace(workspace);
                //geoServerManager.removeAllCoverageStores(workspace, "Z_NWGD_C_*");
            } catch (WorkSpaceNotFoundException e) {
                throw new RuntimeException(e);
//            } catch (CoverageStoreNotFoundException e) {
//                throw new RuntimeException(e);
//            } catch (LayerNotFoundException e) {
//                throw new RuntimeException(e);
            }

//            ArrayList<String> layersList = geoServerReader.getLayersList();
//
//            for (int i = 0; i < layersList.size(); i++) {
//                String layer = layersList.get(i);
//                if (layer.startsWith(workspace + ":") && layer.length() > 40) {
//                    try {
//                        Boolean aBoolean1 = false;
//                        Boolean aBoolean2 = false;
//                        //int i1 = geoServerManager.removeAllCoverageStores(workspace, "SAT_RTD*");
//                        if (geoServerReader.existsLayer(workspace, layer)) {
//                            aBoolean2 = geoServerManager.removeLayer(workspace, layer);
//                        }
//                        if (geoServerReader.existsCoverageStore(workspace, layer)) {
//                            aBoolean1 = geoServerManager.removeCoverageStores(workspace, layer);
//                        }
//                        if (!aBoolean1 || !aBoolean2) {
//                            retFlag = false;
//                        }
//                        log.debug("删除图层" + layer + "完成！");
//                    } catch (WorkSpaceNotFoundException e) {
//                        retFlag = false;
//                        log.debug("删除图层" + layer + "时出错:" + e.getMessage());
//                    } catch (LayerNotFoundException e) {
//                        retFlag = false;
//                        log.debug("删除图层" + layer + "时出错:" + e.getMessage());
//                    } catch (CoverageStoreNotFoundException e) {
//                        retFlag = false;
//                        log.debug("删除图层" + layer + "时出错:" + e.getMessage());
//                    }
//                }
//            }
        } catch (MalformedURLException e) {
            retFlag = false;
            log.debug("删除图层时出错:" + e.getMessage());;
        }

        return retFlag;
    }

    @Override
    public Boolean removeLayer(String workspace, String layer) {
        boolean retFlag = true;
        String wmsServerURL = myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();

        try {
            GeoServerManager geoServerManager = new GeoServerManager(wmsServerURL, username, password);
            GeoServerReader geoServerReader = new GeoServerReader(wmsServerURL, username, password);

            try {
                geoServerManager.removeCoverageStores(workspace, layer);
                //geoServerManager.removeAllCoverageStores(workspace, "Z_NWGD_C_*");
            } catch (WorkSpaceNotFoundException e) {
                throw new RuntimeException(e);
//            } catch (CoverageStoreNotFoundException e) {
//                throw new RuntimeException(e);
//            } catch (LayerNotFoundException e) {
//                throw new RuntimeException(e);
            } catch (CoverageStoreNotFoundException e) {
                throw new RuntimeException(e);
            }
        } catch (MalformedURLException e) {
            retFlag = false;
            log.debug("删除图层时出错:" + e.getMessage());;
        }

        return retFlag;
    }

    @Override
    public Boolean removeLayers(String workspace, String layerpattern) {
        boolean retFlag = true;
        String wmsServerURL = myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();

        try {
            GeoServerManager geoServerManager = new GeoServerManager(wmsServerURL, username, password);
            try {
                geoServerManager.removeAllCoverageStores(workspace, layerpattern);
            } catch (WorkSpaceNotFoundException e) {
                throw new RuntimeException(e);
            } catch (CoverageStoreNotFoundException e) {
                throw new RuntimeException(e);
            } catch (LayerNotFoundException e) {
                throw new RuntimeException(e);
            }
        } catch (MalformedURLException e) {
            retFlag = false;
            log.debug("删除图层时出错:" + e.getMessage());;
        }

        return retFlag;
    }


    /**
     * 保存到数据库t_geoserverinfo表中
     * @param filePathName geotiff文件(带路径和后缀名)
     * @return
     */

    private Long save2DB(String filePathName) {
       return save2DB(filePathName, null, "");
    }


    /**
     * 保存到数据库t_geoserverinfo表中
     * @param filePathName geotiff文件(带路径和后缀名)
     * @param styles 样式名
     * @return
     */

    private Long save2DB(String filePathName, String srs, String styles) {
        String layer = FilenameUtils.getBaseName(filePathName);
        String geoserverUrl =  myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();
        String workspaces = myGeoServerConfig.getWorkspaces();

        String bbox = "";
        String latlonbbox = "";
        LayerSRSbbox coverageNativeBoundingBox = GeoServerPublishUtil.getCoverageNativeBoundingBox(geoserverUrl, username, password, workspaces, layer);

        LayerSRSbbox coverageSRS = GeoServerPublishUtil.getCoverageBoundingBox(geoserverUrl, username, password, workspaces, layer);
        bbox = ""+coverageNativeBoundingBox.getMinx()  + "," + coverageNativeBoundingBox.getMiny() + "," + coverageNativeBoundingBox.getMaxx()  + "," + coverageNativeBoundingBox.getMaxy();
        LayerSRSbbox coverageLatLonBoundingBox = GeoServerPublishUtil.getCoverageLatLonBoundingBox(geoserverUrl, username, password, workspaces, layer);
        latlonbbox = ""+coverageLatLonBoundingBox.getMinx()  + "," + coverageLatLonBoundingBox.getMiny() + "," + coverageLatLonBoundingBox.getMaxx()  + "," + coverageLatLonBoundingBox.getMaxy();
        //坐标系与bbox对应
        String srsNative = coverageSRS.getSrs();
        //保存数据
        Long geoserverpubid = sequenceService.getNextval();
        Geoserverinfo geoserverinfo = new Geoserverinfo();
        geoserverinfo.setId(geoserverpubid);
        geoserverinfo.setGeoserverworkspaces(workspaces);
        geoserverinfo.setUrl(geoserverUrl);
        geoserverinfo.setSrs(srs);
        geoserverinfo.setBbox(bbox);
        geoserverinfo.setLatlonbbox(latlonbbox);
        geoserverinfo.setGeoserverlayers(layer);
        geoserverinfo.setGeoserverstyles(styles);
        geoserverinfoService.save(geoserverinfo);

        return geoserverpubid;
    }


    /**
     * 保存到数据库t_geoserverinfo表中
     * @param filePathName geotiff文件(带路径和后缀名)
     * @param styles 样式名
     * @return 返回id 和 bbox
     */

    private Map<Object, Object> save2DBForTest(String filePathName, String styles) {

       Map<Object, Object> objectObjectLinkedHashMap = new LinkedHashMap<>();

        String layer = FilenameUtils.getBaseName(filePathName);
        String geoserverUrl =  myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();
        String workspaces = myGeoServerConfig.getWorkspaces();

        String bbox = "";
        String latlonbbox = "";
        LayerSRSbbox coverageNativeBoundingBox = GeoServerPublishUtil.getCoverageNativeBoundingBox(geoserverUrl, username, password, workspaces, layer);
        LayerSRSbbox coverageBoundingBox = GeoServerPublishUtil.getCoverageBoundingBox(geoserverUrl, username, password, workspaces, layer);
        bbox = ""+coverageNativeBoundingBox.getMinx()  + "," + coverageNativeBoundingBox.getMiny() + "," + coverageNativeBoundingBox.getMaxx()  + "," + coverageNativeBoundingBox.getMaxy();
        LayerSRSbbox coverageLatLonBoundingBox = GeoServerPublishUtil.getCoverageLatLonBoundingBox(geoserverUrl, username, password, workspaces, layer);
        latlonbbox = ""+coverageLatLonBoundingBox.getMinx()  + "," + coverageLatLonBoundingBox.getMiny() + "," + coverageLatLonBoundingBox.getMaxx()  + "," + coverageLatLonBoundingBox.getMaxy();

        String srs = coverageBoundingBox.getSrs();
        //保存数据
        Long geoserverpubid = sequenceService.getNextval();
        Geoserverinfo geoserverinfo = new Geoserverinfo();
        geoserverinfo.setId(geoserverpubid);
        geoserverinfo.setGeoserverworkspaces(workspaces);
        geoserverinfo.setUrl(geoserverUrl);
        geoserverinfo.setSrs(srs);
        geoserverinfo.setBbox(bbox);
        geoserverinfo.setLatlonbbox(latlonbbox);
        geoserverinfo.setGeoserverlayers(layer);
        geoserverinfo.setGeoserverstyles(styles);
        geoserverinfoService.save(geoserverinfo);

        objectObjectLinkedHashMap.put("id", geoserverpubid);
        objectObjectLinkedHashMap.put("bbox", latlonbbox);


        return objectObjectLinkedHashMap;
    }


    /**
     * 发布geoServer图层
     * @param layer 图层名
     * @param srs 坐标系 （如果数据不为该坐标系统，则通过转换）
     * @param defaultStyle 缺省的样式（访问geoServer时，不带样式的时候，以些样式显示图层）
     * @param filePathName 发布的geotiff文件（带路径和后缀名）
     * @return 成功返回true
     */
    private boolean pubGeoServer(String layer, String srs, String defaultStyle, String filePathName) {

        String geoserverUrl = myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();
        String workspaces = myGeoServerConfig.getWorkspaces();
        GeoTiff geoTiff = null;
        //发布数据
        try {
            MeteGis.init();
            geoTiff = new GeoTiff(filePathName);
            CoordinateReferenceSystem coordinateReferenceSystem = geoTiff.getCoverage().getCoordinateReferenceSystem();
            String nativeSrs = CoordinateUtil.getFirstEPSG(coordinateReferenceSystem);
            double[] nativeSrsbbox = new double[4];
            Envelope2D envelope2D = geoTiff.getCoverage().getEnvelope2D();
            Point p1 = ProjectTrans.xy2xy(envelope2D.getMinX(), envelope2D.getMinY(), nativeSrs, srs);
            Point p2 = ProjectTrans.xy2xy(envelope2D.getMaxX(), envelope2D.getMaxY(), nativeSrs, srs);
            nativeSrsbbox[0] = p1.getX();
            nativeSrsbbox[1] = p1.getY();
            nativeSrsbbox[2] = p2.getX();
            nativeSrsbbox[3] = p2.getY();

            //取经纬度边框
            double[] latlonsbbox = new double[4];
            Point p3 = ProjectTrans.xy2xy(envelope2D.getMinX(), envelope2D.getMinY(), nativeSrs, GeoServerRESTPublisher.DEFAULT_CRS);
            Point p4 = ProjectTrans.xy2xy(envelope2D.getMaxX(), envelope2D.getMaxY(), nativeSrs, GeoServerRESTPublisher.DEFAULT_CRS);
            latlonsbbox[0] = p3.getX();
            latlonsbbox[1] = p3.getY();
            latlonsbbox[2] = p4.getX();
            latlonsbbox[3] = p4.getY();

            geoTiff.close();

            //GSResourceEncoder.ProjectionPolicy.REPROJECT_TO_DECLARED 为投影转换
            //GSResourceEncoder.ProjectionPolicy.FORCE_DECLARED 直接设置（不转换，适合为原数据无坐标系或坐标系错误的数据）
            boolean pubFlag = GeoServerPublishUtil.publishGeoTIFF(geoserverUrl, username, password, workspaces, layer, nativeSrs, srs
                    , GSResourceEncoder.ProjectionPolicy.REPROJECT_TO_DECLARED, defaultStyle, nativeSrsbbox, latlonsbbox, filePathName);
            return pubFlag;
        }  catch (Exception e) {
            log.error("发布数据时出错：" + e.getMessage());
        }finally {
            if (geoTiff != null) {
                geoTiff.close();
            }
        }
        return false;
    }

    /**
     * 发布geoServer图层
     * @param layer 图层名
     * @param filePathName 发布的geotiff文件（带路径和后缀名）
     * @return 成功返回true
     */
    private boolean pubGeoServer(String layer, String filePathName) {
        String geoserverUrl = myGeoServerConfig.getGeoserverurl();
        String username = myGeoServerConfig.getUsername();
        String password = myGeoServerConfig.getPassword();
        String workspaces = myGeoServerConfig.getWorkspaces();

        //发布数据
        try {
            boolean pubFlag = GeoServerPublishUtil.publishGeoTIFF(geoserverUrl, username, password, workspaces, layer, filePathName);
            return pubFlag;
        }  catch (Exception e) {
            log.error("发布数据时出错：" + e.getMessage());
        }
        return false;
    }
}
