package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cuit.solarenergy.entity.LegendTable;
import com.cuit.solarenergy.entity.TSamplingIndex;
import com.cuit.solarenergy.entity.vo.ret.LegendRet;
import com.cuit.solarenergy.mapper.LegendTableMapper;
import com.cuit.solarenergy.mapper.TSamplingIndexMapper;
import com.cuit.solarenergy.service.LegendTableService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author 田俊彪
 * @description 针对表【legend_table(图例表)】的数据库操作Service实现
 * @createDate 2024-09-03 20:34:01
 */
@Service
public class LegendTableServiceImpl extends ServiceImpl<LegendTableMapper, LegendTable>
        implements LegendTableService {

    @Resource
    private LegendTableMapper legendTableMapper;

    @Resource
    private TSamplingIndexMapper tSamplingIndexMapper;

    /**
     * 获取图例数据
     *
     * @param dataType
     * @param dataCode
     * @return
     */
    @Override
    public List<LegendRet> getLegend(String dataType, String dataCode) {
        // 将反演数据转换为代码
        LambdaQueryWrapper<TSamplingIndex> tSamplingIndexLambdaQueryWrapper = new LambdaQueryWrapper<>();
        tSamplingIndexLambdaQueryWrapper.eq(StringUtils.isNotBlank(dataCode), TSamplingIndex::getName, dataCode);
        TSamplingIndex tSamplingIndex = tSamplingIndexMapper.selectOne(tSamplingIndexLambdaQueryWrapper);
        if (tSamplingIndex == null) {
            throw new RuntimeException("反演数据： " + dataCode + " 没有相关图例");
        }
        String code = tSamplingIndex.getCode();
        LambdaQueryWrapper<LegendTable> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(dataType), LegendTable::getDataType, dataType)
                .eq(StringUtils.isNotBlank(code), LegendTable::getDataCode, code)
                .orderByAsc(LegendTable::getMin);
        List<LegendTable> legendTables = legendTableMapper.selectList(queryWrapper);
        if (legendTables != null) {
            List<LegendRet> legendRetList = new ArrayList<>();
            if (legendTables.get(0).getLegendType().equals("分段")) {
                for (int i = 0; i < legendTables.size(); i++) {
                    LegendRet legendRet = new LegendRet();
                    BeanUtils.copyProperties(legendTables.get(i), legendRet);
                    legendRet.setColorR(Integer.parseInt(legendRet.getColor().substring(1, 3), 16));
                    legendRet.setColorG(Integer.parseInt(legendRet.getColor().substring(3, 5), 16));
                    legendRet.setColorB(Integer.parseInt(legendRet.getColor().substring(5, 7), 16));
                    if (i == 0) {
                        legendRet.setMinDisplay(0);
                        legendRet.setMaxDisplay(0);
                    } else {
                        legendRet.setMinDisplay(1);
                        legendRet.setMaxDisplay(0);
                    }
                    legendRetList.add(legendRet);
                }
            } else if (legendTables.get(0).getLegendType().equals("拉伸")) {
                for (int i = 0; i < legendTables.size(); i++) {
                    LegendRet legendRet = new LegendRet();
                    BeanUtils.copyProperties(legendTables.get(i), legendRet);
                    legendRet.setColorR(Integer.parseInt(legendRet.getColor().substring(1, 3), 16));
                    legendRet.setColorG(Integer.parseInt(legendRet.getColor().substring(3, 5), 16));
                    legendRet.setColorB(Integer.parseInt(legendRet.getColor().substring(5, 7), 16));
                    if (i == 0) {
                        legendRet.setMinDisplay(0);
                        legendRet.setMaxDisplay(1);
                    } else if (i == legendTables.size() - 1) {
                        legendRet.setMinDisplay(1);
                        legendRet.setMaxDisplay(0);
                    }else {
                        legendRet.setMinDisplay(1);
                        legendRet.setMaxDisplay(1);
                    }
                    legendRetList.add(legendRet);
                }
            } else {
                for (int i = 0; i < legendTables.size(); i++) {
                    LegendRet legendRet = new LegendRet();
                    BeanUtils.copyProperties(legendTables.get(i), legendRet);
                    legendRet.setColorR(Integer.parseInt(legendRet.getColor().substring(1, 3), 16));
                    legendRet.setColorG(Integer.parseInt(legendRet.getColor().substring(3, 5), 16));
                    legendRet.setColorB(Integer.parseInt(legendRet.getColor().substring(5, 7), 16));
                    legendRet.setMinDisplay(0);
                    legendRet.setMaxDisplay(1);
                    legendRetList.add(legendRet);
                }
            }
            return legendRetList;
        }
        return null;
    }
}



