package com.cuit.solarenergy.service.impl;

import com.cuit.solarenergy.entity.LoginUser;
import com.cuit.solarenergy.entity.User;
import com.cuit.solarenergy.entity.vo.ret.RetResponse;
import com.cuit.solarenergy.entity.vo.ret.RetResult;
import com.cuit.solarenergy.service.LoginService;
import com.cuit.solarenergy.utils.JwtUtil;
import com.cuit.solarenergy.utils.RedisCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-13  11:33
 * @Description: TODO
 * @Version: 1.0
 */
@Service
public class LoginServiceImpl implements LoginService {

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;

    @Override
    public RetResult login(String userName,String password) {
        //AuthenticationManager的authenticate方法进行用户认证
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(userName,password);
        Authentication authenticate = authenticationManager.authenticate(authenticationToken);
        //如果认证没通过，给出对应的提示
        if(Objects.isNull(authenticate)){
            throw new RuntimeException("登录失败");
        }
        //如果认证通过，使用userid生成一个jwt jwt存入RetResult返回
        LoginUser loginUser = (LoginUser)authenticate.getPrincipal();
        String userid = loginUser.getUser().getId().toString();
        String jwt = JwtUtil.createJWT(userid);
        Map<String,Object> map = new HashMap<>();
        map.put("token",jwt);

        //返回用户脱敏信息
        User newUser = new User();
        newUser.setId(loginUser.getUser().getId());
        newUser.setUserName(loginUser.getUser().getUserName());
        newUser.setNickName(loginUser.getUser().getNickName());
        newUser.setSex(loginUser.getUser().getSex());
        newUser.setPhonenumber(loginUser.getUser().getPhonenumber());
        newUser.setEmail(loginUser.getUser().getEmail());
        newUser.setAvatar(loginUser.getUser().getAvatar());
        newUser.setUserType(loginUser.getUser().getUserType());
        map.put("user",newUser);


        //把完整的用户信息存入redis userid作为key
        redisCache.setCacheObject("login:"+userid,loginUser);
        return RetResponse.makeRsp(200, "登陆成功", map);
    }

    @Override
    public RetResult logout() {
        //获取SecurityContextHolder中的用户id
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        Long userid = loginUser.getUser().getId();
        //删除redis中的值
        redisCache.deleteObject("login:"+userid);
        return RetResponse.makeRsp(200, "注销成功");

    }
}
