package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.base.BasePageQuery;
import com.cuit.solarenergy.entity.GeoserverinfoBase;
import com.cuit.solarenergy.entity.TSiteBaseInfo;
import com.cuit.solarenergy.entity.TStreamBaseInfo;
import com.cuit.solarenergy.entity.vo.ret.MonitorGeoURLAndBboxVoRet;
import com.cuit.solarenergy.entity.vo.ret.MonitorSitesVo;
import com.cuit.solarenergy.entity.vo.ret.PageVo;
import com.cuit.solarenergy.mapper.*;
import com.cuit.solarenergy.minio.MinioConfig;
import com.cuit.solarenergy.service.GeoServerPubService;
import com.cuit.solarenergy.service.MonitoringDataService;
import com.cuit.solarenergy.vo.SiteBaseInfo;
import com.cuit.solarenergy.vo.SitesInfoForOne;
import com.cuit.solarenergy.vo.SitesInfosForOne;
import com.cuit.solarenergy.vo.param.SiteInfoParams;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-06-17  09:52
 * @Description: 检测数据实现类
 * @Version: 1.0
 */
@Service
@Slf4j
public class MonitoringDataServiceImpl implements MonitoringDataService {

    @Resource
    private TStreamBaseInfoMapper tStreamBaseInfoMapper;

    @Autowired
    private TSiteBaseInfoMapper tSiteBaseInfoMapper;

    @Autowired
    private TSamplingIndexMapper tSamplingIndexMapper;

    @Autowired
    private TDataOriginMethodMapper tDataOriginMethodMapper;

    @Autowired
    private TSamplingSiteImageMapper tSamplingSiteImageMapper;

    @Autowired
    private GeoserverinfoMapper geoserverinfoMapper;

    @Autowired
    private GeoServerPubService geoServerPubService;

    @Autowired
    GeoserverinfoBaseMapper geoserverinfoBaseMapper;

    @Autowired
    SatelliteSendDataMapper satelliteSendDataMapper;
    @Autowired
    UavSendDataMapper uavSendDataMapper;

    @Resource
    private MinioConfig properties;

    /**
     * 方法功能：根据某一个水质参数查询当前年份所有站点的相关信息
     *
     * @param waterQualityParameter
     * @return
     * @date 2023/6/24 10:17
     * @author Tjb
     */
    @Override
    public List<SitesInfoForOne> getAllSiteForOne(String waterQualityParameter) {
        log.info("根据 {} 水质参数查询当前年份所有站点的相关信息", waterQualityParameter);
        List<SitesInfoForOne> allSiteForOne = tSiteBaseInfoMapper.getAllSiteForOne(waterQualityParameter);
        log.info("根据 {} 水质参数查询当前年份所有站点的相关信息为： {} ", waterQualityParameter, allSiteForOne);
        return allSiteForOne;
    }

    /**
     * 方法功能：获取某一流域指定时间段指定水质参数所有采样点详情
     *
     * @param
     * @param siteInfoParams
     * @param query
     * @return
     * @date 2023/6/24 19:25
     * @author Tjb
     */

    @Override
    public PageVo<SitesInfosForOne> getStreamAllSitesInfosForOne(SiteInfoParams siteInfoParams, BasePageQuery query) {
        log.info("获取 {} 流域 {} - {} 内 {} 参数所有采样点详情", siteInfoParams.getStreamName(), siteInfoParams.getStartTime(), siteInfoParams.getEndTime(), siteInfoParams.getIndexName());
        Page<SitesInfosForOne> page = new Page<>(query.getCurrent(), query.getSize());
        page = tSiteBaseInfoMapper.getStreamAllSitesInfosForOne(page, siteInfoParams);
        return new PageVo<>(page);
    }

    /**
     * 方法功能：根据站点id查询站点基本信息
     *
     * @param siteId
     * @return
     * @date 2023/7/3 10:31
     * @author Tjb
     */

    @Override
    public SiteBaseInfo getSiteBaseInfoById(String siteId) {
        log.info("根据 {} 查询站点基本信息", siteId);
        SiteBaseInfo siteBaseInfoById = tSiteBaseInfoMapper.getSiteBaseInfoById(siteId);
        String[] pictureSplit = null;
        String[] videoSplit = null;
        StringBuilder picStringBuilder = new StringBuilder();
        StringBuilder videoStringBuilder = new StringBuilder();
        if (siteBaseInfoById != null) {
            // 拼接ip和端口
            if (siteBaseInfoById.getSitePictures() != null) {
                pictureSplit = siteBaseInfoById.getSitePictures().split(",");
                for (int i = 0; i < pictureSplit.length; i++) {
                    pictureSplit[i] = getMinioUrl(pictureSplit[i]);
                    picStringBuilder.append(pictureSplit[i]);
                    if (i != pictureSplit.length - 1) {
                        picStringBuilder.append(",");
                    }
                }
            }
            if (siteBaseInfoById.getSiteVideos() != null) {
                videoSplit = siteBaseInfoById.getSiteVideos().split(",");
                for (int i = 0; i < videoSplit.length; i++) {
                    videoSplit[i] = getMinioUrl(videoSplit[i]);
                    videoStringBuilder.append(videoSplit[i]);
                    if (i != videoSplit.length - 1) {
                        videoStringBuilder.append(",");
                    }
                }
            }
            siteBaseInfoById.setSitePictures(String.valueOf(picStringBuilder));
            siteBaseInfoById.setSiteVideos(String.valueOf(videoStringBuilder));

            log.info("根据 {} 查询站点基本信息为： {} ", siteId, siteBaseInfoById);
        }
        return siteBaseInfoById;
    }

    /**
     * 方法功能：获取站点的所有图片
     *
     * @param siteId
     * @param query
     * @return
     * @date 2023/7/3 11:40
     * @author Tjb
     */

    @Override
    public PageVo<String> getSitePictures(String siteId, BasePageQuery query) {
        log.info("获取站点  {}  的所有图片", siteId);
        Page<String> page = new Page<>(query.getCurrent(), query.getSize());
        page = tSamplingSiteImageMapper.getSitePictures(page, siteId);
        return new PageVo<>(page);
    }

    /**
     * 方法功能：获取站点的所有视频
     *
     * @param siteId
     * @param query
     * @return
     * @date 2023/7/3 11:40
     * @author Tjb
     */

    @Override
    public PageVo<String> getSiteVideos(String siteId, BasePageQuery query) {
        log.info("获取站点  {}  的所有视频", siteId);
        Page<String> page = new Page<>(query.getCurrent(), query.getSize());
        page = tSamplingSiteImageMapper.getSiteVideos(page, siteId);
        return new PageVo<>(page);
    }

    /**
     * 方法功能：获取监测数据首页指定流域指定水质参数当前年的站点信息List
     * 有可能当前没有站点，所以修改为取所有的已监测的站点
     *
     * @param streamName 流域名称
     * @param indexName  水质参数名称
     * @return
     * @date 2023/7/15 15:06
     * @author Tjb
     */

    @Override
    public List<MonitorSitesVo> getMonitorSitesList(String streamName, String indexName) {
        log.info("获取监测数据首页 {} 流域水质参数为 {} 当前年的站点信息List", streamName, indexName);
        //获取当前年
        Calendar calendar = Calendar.getInstance();
//        int year = calendar.get(Calendar.YEAR);
        List<MonitorSitesVo> monitorSitesVoList = tSiteBaseInfoMapper.getMonitorSitesList(streamName, indexName);
        log.info("获取监测数据首页 {} 流域水质参数为 {} 当前年的站点信息List信息为：{}", streamName, indexName, monitorSitesVoList);
        return monitorSitesVoList;
    }

    /**
     * 方法功能：返回图层URL和bbox
     *
     * @param streamCode 流域代码
     * @param typeCode   大类代码
     * @param dataCode   数据代码
     * @return
     * @date 2023/8/10 9:29
     * @author Tjb
     */

    @Override
    public List<MonitorGeoURLAndBboxVoRet> getGeoserverIdAndBbox(String streamCode, String typeCode, String dataCode) throws Exception {
        String uavOrSat;
        String band;
        List<MonitorGeoURLAndBboxVoRet> geoServerPubList = new LinkedList<>();
        //有一些数据是全范围的，不分流域
        if (!StringUtils.isEmpty(streamCode)) {
            //1. 校验流域代码是否存在
            TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(new LambdaQueryWrapper<TStreamBaseInfo>().eq(TStreamBaseInfo::getStreamCode, streamCode));
            if (tStreamBaseInfo == null) {
                throw new Exception("不存在该流域代码！");
            }
        }

        //2. 校验数据获取方式
        switch (typeCode) {
            case "UAV":
                //无人机
                geoServerPubList = uavSendDataMapper.selectByStreamCodeDataCode(streamCode, dataCode);
                for (int i = 0; i < geoServerPubList.size(); i++) {
                    geoServerPubList.get(i).setGeoserverURL(geoServerPubService.getMapBox3857RasterWmsUrl(geoServerPubList.get(i).getId()));
                }
                break;
            case "SAT":
                //卫星
                geoServerPubList = satelliteSendDataMapper.selectByStreamCodeDataCode(streamCode, dataCode);
                for (int i = 0; i < geoServerPubList.size(); i++) {
                    geoServerPubList.get(i).setGeoserverURL(geoServerPubService.getMapBox3857RasterWmsUrl(geoServerPubList.get(i).getId()));
                }
                break;
            case "OTHER":

                List<GeoserverinfoBase> geoserverinfoBases = geoserverinfoBaseMapper.selectByStreamCodeDataCode(streamCode, dataCode);
                for (int i = 0; i < geoserverinfoBases.size(); i++) {
                    MonitorGeoURLAndBboxVoRet monitorGeoURLAndBboxVoRet = new MonitorGeoURLAndBboxVoRet();
                    monitorGeoURLAndBboxVoRet.setId(geoserverinfoBases.get(i).getId());
                    monitorGeoURLAndBboxVoRet.setLatLonBbox(geoserverinfoBases.get(i).getBbox());
                    //monitorGeoURLAndBboxVoRet.setGeoserverURL(geoserverinfoBases.get(i).getUrl());
                    monitorGeoURLAndBboxVoRet.setGeoserverURL(geoServerPubService.getMapBox3857RasterWmsUrlByBaseInfo(geoserverinfoBases.get(i).getId()));
                    geoServerPubList.add(monitorGeoURLAndBboxVoRet);
                }
                //其它
                break;
            default:
                throw new Exception("数据获取方式应该以 UAV、SAT或OTHER！");
        }

        return geoServerPubList;
    }

    @Override
    public Boolean addStream(String streamCode, String streamName) throws Exception {
        // 检查唯一性
        LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TStreamBaseInfo::getStreamCode, streamCode)
                .eq(TStreamBaseInfo::getStreamName, streamName);
        TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(queryWrapper);
        if (tStreamBaseInfo != null) {
            throw new Exception("流域已存在！");
        } else {
            TStreamBaseInfo newStream = new TStreamBaseInfo();
            newStream.setStreamCode(streamCode);
            newStream.setStreamName(streamName);
            int insert = tStreamBaseInfoMapper.insert(newStream);
            return insert != 0;
        }
    }

    @Override
    public Boolean addSite(String stationNo, String streamCode, Double lon, Double lat, String relativeSiteLocation, Double relativeSiteDistance, Integer distance) throws Exception {
        // 检查唯一性
        LambdaQueryWrapper<TSiteBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TSiteBaseInfo::getStationNo, stationNo)
                .eq(TSiteBaseInfo::getStreamCode, streamCode);
        TSiteBaseInfo oldSite = tSiteBaseInfoMapper.selectOne(queryWrapper);
        if (oldSite != null) {
            throw new Exception("站点已存在！");
        } else {
            TSiteBaseInfo newSite = new TSiteBaseInfo();
            newSite.setStationNo(stationNo);
            newSite.setStreamCode(streamCode);
            newSite.setLon(lon);
            newSite.setLat(lat);
            newSite.setRelativeSiteLocation(relativeSiteLocation);
            newSite.setRelativeSiteDistance(relativeSiteDistance);
            newSite.setDistance(distance);
            int insert = tSiteBaseInfoMapper.insert(newSite);
            return insert != 0;
        }
    }

    @Override
    public Boolean editStream(String streamCode, String streamName) {
        int edit = tStreamBaseInfoMapper.editStream(streamCode, streamName);
        return edit != 0;
    }

    @Override
    public Boolean editSite(String stationNo, Double lon, Double lat, String relativeSiteLocation, Double relativeSiteDistance, Integer distance) {
        int edit = tSiteBaseInfoMapper.editSite(stationNo, lon, lat, relativeSiteLocation, relativeSiteDistance, distance);
        return edit != 0;
    }

    private String getMinioUrl(String minioID) {
        String url = properties.getVisitPoint();
        if (StringUtils.isEmpty(minioID)) {
            return minioID;
        }
        if (!(url.endsWith(File.separator) && url.endsWith(File.pathSeparator))) {
            url = url + File.separator;
        }
        String miniourl = url + minioID;
//        String miniourl = url + minioID.substring(25);
        return miniourl;
    }
}
